﻿#region Using
using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Layouts;
using Infragistics.Controls.Layouts.Primitives;
using System.Collections.Generic;
#endregion Using
namespace CustomCommanding
{
    public enum CustomCommands
    {
        Close,
        CloseAllSiblings,
    };

    public class CustomCommandSource : XamDockPaneCommandSource
    {
        public new CustomCommands CommandType { get; set; }

        protected override ICommand ResolveCommand()
        {
            switch (CommandType)
            {
                case CustomCommands.Close:
                    return new CloseCommand();
                case CustomCommands.CloseAllSiblings:
                    return new CloseAllSiblingsCommand();
            }

            return null;
        }
    }

    public class CloseCommand : XamDockPaneCommandBase
    {
        protected override void ExecuteCommand(PaneBase pane)
        {
            if (pane != null && pane.XamDockManager != null)
            {
                if (pane is ContentPane)
                {
                    ContentPane p = pane as ContentPane;
                    p.Content = null;
                }
                // Close the pane.  It won't be placed in the ClosedPanes collection.
                pane.ClosePane();
            }
        }
        protected override bool CanExecuteCommand(PaneBase pane)
        {
            // Always allow execution.
            return true;
        }
    }

    public class CloseAllSiblingsCommand : XamDockPaneCommandBase
    {
        protected override void ExecuteCommand(PaneBase pane)
        {
            if (pane != null && pane.XamDockManager != null)
            {
                // The top-right close button actually closes the TabGroupPane in this case.
                if (pane is TabGroupPane)
                {
                    TabGroupPane tgp = pane as TabGroupPane;
                    List<ContentPane> tgp_children = new List<ContentPane>(tgp.Panes);
                    foreach (ContentPane cp in tgp_children)
                    {
                        if (!cp.Name.StartsWith("cpDummy"))
                        {
                            cp.Content = null;
                            // Close the pane.  It won't be placed in the ClosedPanes collection.
                            cp.ClosePane();
                        }
                    }
                }
                else
                {
                    // Remove all panes from the list.
                    TabGroupPane tgp = pane.OwnerPane as TabGroupPane;

                    // I create another list so that I can close all children in the TabGroupPane.  Normally when there is 1
                    // pane left the TagGroupPane will kill itself which will cause the Panes collection to become empty which
                    // means the last pan will get away.
                    List<ContentPane> tgp_children = new List<ContentPane>(tgp.Panes);
                    foreach (ContentPane cp in tgp_children)
                    {
                        if (!cp.Name.StartsWith("cpDummy"))
                        {
                            cp.Content = null;
                            // Close the pane.  It won't be placed in the ClosedPanes collection.
                            cp.ClosePane();
                        }
                    }
                }
            }
        }

        protected override bool CanExecuteCommand(PaneBase pane)
        {
            // Always allow execution.
            return true;
        }
    }
}
