
#define SUPPORT_UNDO_REDO
#define SUPPORT_TEST_LIST_BANDS

#region Using
using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using WMAApp.ViewModels;
using System.Collections.Generic;
using WMAApp.Web.Models;
using Infragistics.Undo;
using System.ComponentModel;
using SimpleMvvmToolkit;
using System.Windows.Data;

#endregion Using


//todo: figure out how and where to encapsulate all of this...
//todo: might be overkill or too much overhead to derrive these from WMAVMBase. Maybe make lighter "notify" base class?
namespace WMAApp.Helpers
{


    public enum EPDBSaveTypes
    {
        eALL,       // Save all tests
        eSELECTED   // Save only selected tests
    }
    enum EPDCompleteTypes
    {
        eGET_PDB_COMPLETE,
        eDELETE_PDB_COMPLETE,
        eDELETE_PDB_BEFORE_INSERT_COMPLETE,
        eINSERT_PDB_COMPLETE,
        //composition eINSERT_PDB_DEFN_COMPLETE,
        eINSERT_PDB_DATA_COMPELTE, 
        //composition eGET_PDB_DEFN_SAKEY_COMPLETE

    }
   
#if SUPPORT_UNDO_REDO 

    public class ObservableModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(String _sInfo)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(_sInfo));
            }
        }
    }

    // A base class that supports undo/redo of property changes.
    public class UndoPropertyChange : ObservableModel
    {
        private object _Owner;
        internal object Owner
        {
            get { return _Owner; }
            set { _Owner = value; }
        }

        protected bool SetField<T>(ref T _member, T _newValue, string _propertyName)
        {
            if (EqualityComparer<T>.Default.Equals(_member, _newValue))
                return false;

            if (_Owner != null)
                UndoManager.FromReference(_Owner).AddPropertyChange(this, _propertyName, _member, _newValue);

            _member = _newValue;
            //done by virtue of above line of code and inheriting from viewmodelbase     this.NotifyPropertyChanged(propertyName);
            return true;
        }       
    }
 
    public class ProductDataUR : UndoPropertyChange
    {
        public string sFamilyId { get; set; }
        public DateTime dtTime { get; set; } 
        public string sStand { get; set; }
        public string sModel { get; set; }
        public string sSerial { get; set; }
        public string sStatus { get; set; }

        public ProductDataUR() {}
           
#if SUPPORT_TEST_LIST_BANDS
        public ProductDataUR(string f, DateTime tm, string st, string mc, string sc, string sta, 
                               List<PRODUCT_DATA_STAT> _lstPds, List<PRODUCT_DATA_REJ> _lstRejects )
        {
            sFamilyId = f;
            dtTime = tm;
            sStand = st;
            sModel = mc;
            sSerial = sc;
            sStatus = sta;

            m_colPds = new ObservableCollection<ProductDataStat>();
            string sDesc = string.Empty;
            string sLongDesc = string.Empty;          

            foreach (PRODUCT_DATA_STAT pds in _lstPds)
            {
                sLongDesc = sDesc = "some description";
                if (pds.PRODUCT_DATA_STAT_DESC != null)
                {
                    sDesc = pds.PRODUCT_DATA_STAT_DESC.SHORT_DESC_TEXT; 
                    sLongDesc = pds.PRODUCT_DATA_STAT_DESC.LONG_DESC_TEXT; 
                }
                m_colPds.Add(new ProductDataStat( pds.PRODUCT_DATA_STAT_ID, sDesc, sLongDesc ));
            }


            m_colRejects = new ObservableCollection<ProductDataRej>();
            sDesc = string.Empty;
            foreach (PRODUCT_DATA_REJ pdr in _lstRejects)
            {
                sDesc = "some reject description";
                //todo if (pdr.xxxRejectText != null)
                //todo    sDesc = pdr.xxxRejectText;  
                m_colRejects.Add(new ProductDataRej(pdr.REJ_ID, sDesc));
            }
        }
        public ProductDataUR(string f, DateTime tm, string st, string mc, string sc, string sta,
                             Collection<ProductDataStat> _colPds, Collection<ProductDataRej> _colRejects)
        {
            sFamilyId = f;
            dtTime = tm;
            sStand = st;
            sModel = mc;
            sSerial = sc;
            sStatus = sta;
            m_colPds = new ObservableCollection<ProductDataStat>(_colPds);
            m_colRejects = new ObservableCollection<ProductDataRej>(_colRejects);
        }
        private Collection<ProductDataStat> m_colPds;
        public Collection<ProductDataStat> colStatusList { get { return m_colPds; } }
        private Collection<ProductDataRej> m_colRejects;
        public Collection<ProductDataRej> colRejList { get { return m_colRejects; } }
#else
        public ProductDataUR(string f, DateTime tm, string st, string mc, string sc, string sta)
        {
            sFamilyId = f;
            dtTime = tm;
            sStand = st;
            sModel = mc;
            sSerial = sc;
            sStatus = sta;
        }
#endif
    }
   
    // A derived class from the Undo\Redo framework ObservableCollectionExtendedWithUndo class
    public class ProductDataUROC : ObservableCollectionExtendedWithUndo<ProductDataUR>
    {
        public ProductDataUROC(UndoManager undoManager)
            : base(undoManager)
        {
            undoManager.RegisterReference(this);
        }
        protected override void InsertItem(int index, ProductDataUR item)
        {
            item.Owner = this;
            base.InsertItem(index, item);
        }
        protected override void RemoveItem(int index)
        {
            ProductDataUR item = this[index];
            item.Owner = null;

            base.RemoveItem(index);
        }
    }

#endif

    public class ProductData
    {
        public string sFamilyId { get; set; }
        public DateTime dtTime { get; set; }
        public string sStand { get; set; }
        public string sModel { get; set; }
        public string sSerial { get; set; }
        public string sStatus { get; set; }
        
        public ProductData() { }
#if SUPPORT_TEST_LIST_BANDS
        public ProductData(string f, DateTime tm, string st, string mc, string sc, string sta,
                           List<PRODUCT_DATA_STAT> _lstPds, List<PRODUCT_DATA_REJ> _lstRejects)
        {
            sFamilyId = f;
            dtTime = tm;
            sStand = st;
            sModel = mc;
            sSerial = sc;
            sStatus = sta;

            m_colPds = new ObservableCollection<ProductDataStat>();
            string sDesc = string.Empty;
            string sLongDesc = string.Empty;
            foreach (PRODUCT_DATA_STAT pds in _lstPds)
            {
                sLongDesc = sDesc = "some description";
                if (pds.PRODUCT_DATA_STAT_DESC != null)
                {
                    sDesc = pds.PRODUCT_DATA_STAT_DESC.SHORT_DESC_TEXT; 
                    sLongDesc = pds.PRODUCT_DATA_STAT_DESC.LONG_DESC_TEXT; 
                }
                m_colPds.Add(new ProductDataStat( pds.PRODUCT_DATA_STAT_ID, sDesc, sLongDesc ));
            }

            m_colRejects = new ObservableCollection<ProductDataRej>();
            sDesc = string.Empty;
            foreach (PRODUCT_DATA_REJ pdr in _lstRejects)
            {
                sDesc = "some reject description";
                //todo if (pdr.xxxRejectText != null)
                //todo    sDesc = pdr.xxxRejectText;  
                m_colRejects.Add(new ProductDataRej(pdr.REJ_ID, sDesc));
            }
        }
        public ProductData(string f, DateTime tm, string st, string mc, string sc, string sta,
                            Collection<ProductDataStat> _colPds, Collection<ProductDataRej> _colRejects)
        {
            sFamilyId = f;
            dtTime = tm;
            sStand = st;
            sModel = mc;
            sSerial = sc;
            sStatus = sta;
            m_colPds = new ObservableCollection<ProductDataStat>(_colPds);
            m_colRejects = new ObservableCollection<ProductDataRej>(_colRejects);
        }
        private Collection<ProductDataStat> m_colPds;
        public Collection<ProductDataStat> colStatusList { get { return m_colPds; } }
        private Collection<ProductDataRej> m_colRejects;
        public Collection<ProductDataRej> colRejList { get { return m_colRejects; } }
#else
        public ProductData(string f, DateTime tm, string st, string mc, string sc, string sta)
        {
            sFamilyId = f;
            dtTime = tm;
            sStand = st;
            sModel = mc;
            sSerial = sc;
            sStatus = sta;
        }
#endif


    }
   
    // todo: how share this across client/server?
    //public class PartData {
    //    public PartData(string _sModel, string _sSerial)
    //    {
    //        Model = _sModel;
    //        Serial = _sSerial;
    //    }
    //    public string Model { get; set; }
    //    public string Serial { get; set; }
    //}

    // Stand Definition Class

    public class StandDef : WMAVMBase<StandDef>
    {
        public string sStandCode { get; set; }
        public string sBtCode { get; set; }
        public int iStandNumber { get; set; }
        public bool? bSelected;

        // this is exposed so that we can pick up when the user selects or deselects an item from a list

        public bool? IsSelected
        {
            get
            {
                return bSelected;
            }
            set
            {
                bSelected = value;
                NotifyPropertyChanged(m => m.IsSelected);
            }
        }

        public StandDef()
        {
            sStandCode = "";
            sBtCode = "";
            iStandNumber = 0;
            IsSelected = false;
        }

        public StandDef(string sc, string bt, int sn, bool boSelect)
        {
            sStandCode = sc;
            sBtCode = bt;
            iStandNumber = sn;
            IsSelected = boSelect;
        }
    }

    //// Family Definition Class


    public class FamilyDef
    {
        public string sFamilyName { get; set; }
        public int iFamilyId { get; set; }

        public FamilyDef(string name, int id)
        {
            sFamilyName = name;
            iFamilyId = id;
        }
    }

    // Model Definition Class
    public class ModelDef : WMAVMBase<ModelDef>
    {
        public string sModelName { get; set; }
        public string sModelText { get; set; }
        public bool? bSelected;

        // this is exposed so that we can pick up when the user selects or deselects an item from a list

        public bool? bIsSelected
        {
            get
            {
                return bSelected;
            }
            set
            {
                bSelected = value;
                NotifyPropertyChanged(m => m.bIsSelected);//OnPropertyChanged("IsSelected");
            }
        }

        public ModelDef()
        {
            sModelName = "";
            sModelText = "";
            bIsSelected = false;
        }

        public ModelDef(string name, string text, bool boSelect)
        {
            sModelName = name;
            sModelText = text;
            bIsSelected = boSelect;
        }
    }

    public class ProductDataStat
    {
        public decimal dId { get; set; }
        public string sShortDesc { get; set; }
        public string sLongDesc { get; set; }
        public ProductDataStat() { }
        public ProductDataStat(decimal id, string sd, string ld)
        {
            dId = id;
            sShortDesc = sd;
            sLongDesc = ld;
        }
    }

#if SUPPORT_TEST_LIST_BANDS
    public class ProductDataRej
    {
        public decimal dId { get; set; }
        public string sDesc { get; set; }
        public ProductDataRej() { }
        public ProductDataRej(decimal _dId, string _sDesc)
        {
            dId = _dId;
            sDesc = _sDesc;
        }
    }
#endif

    public class DbConnection
    {
        public string sConnString { get; set; }
        public string sName { get; set; }
        public bool boSelected { get; set; }
    }

    //public class SelectedTest
    //{
    //    public SelectedTest(DateTime _time, string _stand)
    //    {
    //        Time = _time;
    //        Stand = _stand;
    //    }
    //    public DateTime Time { get; set; }
    //    public string Stand { get; set; }
    //}
    public class WMAMethods {
        public DependencyObject FindChild(DependencyObject _parent, Type _tType)
        {
           if (_parent.GetType() == _tType)
              return _parent;
           for (int i = 0; i < VisualTreeHelper.GetChildrenCount(_parent); i++)
           {
              DependencyObject child = VisualTreeHelper.GetChild(_parent, i);
              if (child.GetType() == _tType)
                 return child;
      
              child = FindChild(child, _tType);
              if (child != null)
                 return child;
           }
           return null;
        }
    }
    // Custom comparer for parts
    public class PartComparer : IEqualityComparer<ProductData>
    {
        // Products are equal if their model and product serial are equal. //todo: check family too?
        public bool Equals(ProductData PD1, ProductData PD2)
        {

            //Check whether the compared objects reference the same data.
            if (Object.ReferenceEquals(PD1, PD2)) return true;

            //Check whether any of the compared objects is null.
            if (Object.ReferenceEquals(PD1, null) || Object.ReferenceEquals(PD2, null))
                return false;

            //Check whether the products' properties are equal.
            return PD1.sFamilyId == PD2.sFamilyId && PD1.sModel == PD2.sModel && PD1.sSerial == PD2.sSerial;
        }

        // If Equals() returns true for a pair of objects 
        // then GetHashCode() must return the same value for these objects.

        public int GetHashCode(ProductData _PD)
        {
            ////Check whether the object is null
            //if (Object.ReferenceEquals(_PD, null)) return 0;    

            ////Get hash code for the MODEL_CODE field if it is not null.
            //int hashFamily = _PD.sFamilyId == null ? 0 : _PD.sFamilyId.GetHashCode();

            ////Get hash code for the MODEL_CODE field if it is not null.
            //int hashModel = _PD.sModel == null ? 0 : _PD.sModel.GetHashCode();

            ////Get hash code for the SERIAL_CODE field.
            //int hashSerial = _PD.sSerial.GetHashCode();

            ////Calculate the hash code.
            //return hashFamily ^ hashModel ^ hashSerial;

            int hc = _PD.GetHashCode();
            return hc;
        }

    }
}
