﻿using System;
using System.Net;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using SimpleMvvmToolkit;
using SimpleMvvmToolkit.ModelExtensions;
using WMAApp.Web.Models;
using MathNet.Numerics.Statistics;
using System.Collections.Generic;
using WMAApp.WMASharedClasses;

namespace WMAApp.ViewModels.Charts
{
    public enum ChartTypes
    {
        ctTrend,
        ctHistogram,
        ctXBarR,
        ctScatter
    }


    public class ChartsVM : WMAVMBase<ChartsVM>
    {
        ChartTypes _ctType;

        // default constructor
        public ChartsVM()
        {
            _ocTPData = new ObservableCollection<TPData>();
            _ocLowLimit = new ObservableCollection<LimitData>();
            _ocHighLimit = new ObservableCollection<LimitData>();
            _ocData = new ObservableCollection<HistogramColumns>();
            _iNumberBars = 50;
        }

        public ChartsVM(ChartTypes ctChartType) : this()
        {
            _ctType = ctChartType;
        }

        int _iNumberBars;
        public int iNumberBars
        {
            get
            {
                return _iNumberBars;
            }

            set
            {
                if (value != _iNumberBars)
                {
                    _iNumberBars = value;
                    NotifyPropertyChanged(m => m.iNumberBars);
                }
            }
        }

        ObservableCollection<HistogramColumns> _ocData;
        public ObservableCollection<HistogramColumns> ocData
        {
            get
            {
                return _ocData;
            }
            set
            {
                if (value != _ocData)
                {
                    _ocData = value;
                    NotifyPropertyChanged(m => m.ocData);
                }
            }
        }

        ObservableCollection<TPData> _ocTPData;
        public ObservableCollection<TPData> ocTPData
        {
            get
            {
                return _ocTPData;
            }
            set
            {
                if (value != _ocTPData)
                {
                    _ocTPData = value;
                    NotifyPropertyChanged(m => m.ocTPData);
                }
            }
        }

        ObservableCollection<LimitData> _ocLowLimit;
        public ObservableCollection<LimitData> ocLowLimit
        {
            get
            {
                return _ocLowLimit;
            }
            set
            {
                if (value != _ocLowLimit)
                {
                    _ocLowLimit = value;
                    NotifyPropertyChanged(m => m.ocLowLimit);
                }
            }
        }

        ObservableCollection<LimitData> _ocHighLimit;
        public ObservableCollection<LimitData> ocHighLimit
        {
            get
            {
                return _ocHighLimit;
            }
            set
            {
                if (value != _ocHighLimit)
                {
                    _ocHighLimit = value;
                    NotifyPropertyChanged(m => m.ocHighLimit);
                }
            }
        }

        public List<string> GetFilterItems(string sFilterType)
        {
            List<string> sFilter = new List<string>();

            // build a list of the stands
            foreach (TPData tp in ocTPData)
            {
                if (sFilterType == "S")
                {
                    if (!sFilter.Contains(tp.sStandCode))
                    {
                        sFilter.Add(tp.sStandCode);
                    }
                }
                else if (sFilterType == "M")
                {
                    if (!sFilter.Contains(tp.sModel))
                    {
                        sFilter.Add(tp.sModel);
                    }
                }
            }

            sFilter.Sort();
            return sFilter;
        }

        public void vStoreData(ObservableCollection<TPData> _Data)
        {
            ocTPData.Clear();

            switch (_ctType)
            {
                case ChartTypes.ctTrend:
                    foreach (TPData tp in _Data)
                    {
                        TPData newTp = new TPData();
                        newTp = tp;
                        ocTPData.Add(newTp);
                    }
                    break;

                case ChartTypes.ctHistogram:
                    
                    foreach (TPData tp in _Data)
                    {
                        TPData newCatData = new TPData();
                        newCatData = tp;
                        ocTPData.Add(newCatData);
                    }

                    break;

                case ChartTypes.ctScatter:
                    break;

                case ChartTypes.ctXBarR:
                    break;
            }
        }
        public void vStoreLimitData(string sType, ObservableCollection<LimitData> _lData)
        {
            if (sType == "L")
                ocLowLimit.Clear();
            else
                ocHighLimit.Clear();

            foreach (LimitData ld in _lData)
            {
                LimitData newLd = new LimitData();
                newLd = ld;

                if (sType == "L")
                {
                    ocLowLimit.Add(newLd);
                }
                else
                {
                    ocHighLimit.Add(newLd);
                }
            }
        }
    }
}
