﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Threading;
using System.Collections.Generic;
// Toolkit extension methods
using SimpleMvvmToolkit.ModelExtensions;
using WMAApp;
using SimpleMvvmToolkit;
using WMAApp.Helpers;
using WMAApp.ViewModels;
using WMAApp.Other;

namespace WMAApp.ViewModels
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvmprop</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// </summary>
    public class ConfirmVM : WMAVMBase<ConfirmVM>
    {
        #region Initialization and Cleanup

        //App currApp;
        private ChildWindow popUp;

        // Default constructor
        public ConfirmVM()
        {
            // Register to get notified when to show myself.
            this.RegisterToReceiveMessages(MessageTokens.sSHOW_OKCANCELMSGBOX, Show);
            //MessageBus.Default.Register(MessageTokens.sSHOW_DBCONNECTIONS, Show);

            SetPopUpCommand = new DelegateCommand<object>(SetPopUp, CanSetPopUp);
            OkButtonCommand = new DelegateCommand(OkButton, CanOkButton);
            YesButtonCommand = new DelegateCommand(YesButton, CanYesButton);
            NoButtonCommand = new DelegateCommand(NoButton, CanNoButton);
            CancelButtonCommand = new DelegateCommand(CancelButton, CanCancelButton);
        }

        #endregion

        #region Notifications
        // Communication between viewModel and view.

        // Add events to notify the view or obtain data from the view
        public event EventHandler<NotificationEventArgs<Exception>> ErrorNotice;

        // notify view when to show myself
        public event EventHandler<NotificationEventArgs> ShowNotice;

        #endregion

        #region Properties

        private bool isBusy;
        public bool IsBusy
        {
            get { return isBusy; }
            set
            {
                isBusy = value;
                NotifyPropertyChanged(m => m.IsBusy);
            }
        }

        private string m_sMessage;
        public string sMessage
        {
            get
            {
                return m_sMessage;
            }

            set
            {
                m_sMessage = value;
                NotifyPropertyChanged(msg => msg.m_sMessage);
            }
        }

        #endregion

        // Commands
        #region Commands

        private DelegateCommand<object> setPopUpCommand;
        public DelegateCommand<object> SetPopUpCommand
        {
            get
            {
                return setPopUpCommand;
            }
            private set { setPopUpCommand = value; }
        }
        public void SetPopUp(object param)
        {
            popUp = (ChildWindow)param;
        }
        private bool CanSetPopUp(object param)
        {
            return true;
        }

        private DelegateCommand okButtonCommand;
        public DelegateCommand OkButtonCommand
        {
            get
            {
                return okButtonCommand;
            }
            private set { okButtonCommand = value; }
        }
        public void OkButton()
        {
            // Broadcast notification to other viewmodel listeners that user changed db selection.
//            SendMessage(MessageTokens.sDB_CONNECTION_CHANGED,
//                        new NotificationEventArgs<ObservableCollection<DbConnection>>(null, Connections));

        }
        private bool CanOkButton()
        {
            return true;
        }

        private DelegateCommand yesButtonCommand;
        public DelegateCommand YesButtonCommand
        {
            get
            {
                return yesButtonCommand;
            }
            private set { yesButtonCommand = value; }
        }
        public void YesButton()
        {
            // Broadcast notification to other viewmodel listeners that user changed db selection.
            //            SendMessage(MessageTokens.sDB_CONNECTION_CHANGED,
            //                        new NotificationEventArgs<ObservableCollection<DbConnection>>(null, Connections));

        }
        private bool CanYesButton()
        {
            return true;
        }

        private DelegateCommand noButtonCommand;
        public DelegateCommand NoButtonCommand
        {
            get
            {
                return noButtonCommand;
            }
            private set { noButtonCommand = value; }
        }
        public void NoButton()
        {
            // Broadcast notification to other viewmodel listeners that user changed db selection.
            //            SendMessage(MessageTokens.sDB_CONNECTION_CHANGED,
            //                        new NotificationEventArgs<ObservableCollection<DbConnection>>(null, Connections));

        }
        private bool CanNoButton()
        {
            return true;
        }

        private DelegateCommand cancelButtonCommand;
        public DelegateCommand CancelButtonCommand
        {
            get
            {
                return cancelButtonCommand;
            }
            private set { cancelButtonCommand = value; }
        }
        public void CancelButton()
        {
            popUp.DialogResult = false;
        }
        private bool CanCancelButton()
        {
            return true;
        }

        #endregion


        #region Methods

        public void Show(object sender, NotificationEventArgs e)
        {
            //tell view to show           
            Notify(ShowNotice, new NotificationEventArgs());

        }

        // methods that will be called by the view

        #endregion

        #region Completion Callbacks

        #endregion

        #region Helpers

        // Helper method to notify View of an error
        private void NotifyError(string _sMessage, Exception _exError)
        {
            // Notify view of an error
            Notify(ErrorNotice, new NotificationEventArgs<Exception>(_sMessage, _exError));
        }

        #endregion
    }
}
