﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Threading;
using System.Collections.Generic;
// Toolkit extension methods
using SimpleMvvmToolkit.ModelExtensions;
using WMAApp;
using SimpleMvvmToolkit;
using WMAApp.Helpers;
using WMAApp.Web.Models;
using WMAApp.ViewModels;
using WMAApp.Services.Connection;
using WMAApp.Other;

namespace WMAApp.ViewModels.Connection
{
    /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvmprop</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// </summary>
    public class ConnectStatusVM : WMAVMBase<ConnectStatusVM>
    {
        #region Initialization and Cleanup
     
        // Default constructor
        public ConnectStatusVM()
        {
            // Register to get notified when db selection changed.
            //this.RegisterToReceiveMessages(MessageTokens.sDB_CONNECTION_CHANGED, DbSelectionChanged);
            this.RegisterToReceiveMessages<ObservableCollection<DbConnection>>(MessageTokens.sDB_CONNECTION_CHANGED, DbSelectionChanged);
        }
       
        #endregion

        #region Notifications
        // Communication between viewModel and view.

        // Add events to notify the view or obtain data from the view
        public event EventHandler<NotificationEventArgs<Exception>> ErrorNotice;
        // notify view when dbselectionchanged
        public event EventHandler<NotificationEventArgs> DbSelectionChangedNotice;

        #endregion

        #region Properties

        public System.ServiceModel.DomainServices.Client.ApplicationServices.AuthenticationService Authentication { 
            get {
                return WebContext.Current.Authentication;
            }
        }
        #endregion

        // Commands
        #region Commands     
       
        #endregion

        #region Methods
        // public void DbSelectionChanged(object sender, NotificationEventArgs e)
        public void DbSelectionChanged(object sender, NotificationEventArgs<ObservableCollection<DbConnection>> e)       
        {
            Notify(DbSelectionChangedNotice, new NotificationEventArgs());

        }

        // methods that will be called by the view
        public void ShowDbSelections()
        {
            MessageBus.Default.Notify(MessageTokens.sSHOW_DBCONNECTIONS, this, new NotificationEventArgs());             
        }
       
        #endregion

        #region Completion Callbacks


        #endregion

        #region Helpers

        // Helper method to notify View of an error
        private void NotifyError(string _sMessage, Exception _exError)
        {
            // Notify view of an error
            Notify(ErrorNotice, new NotificationEventArgs<Exception>(_sMessage, _exError));
        }

        #endregion
    }
}

