﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Threading;
using System.Collections.Generic;
// Toolkit extension methods
using SimpleMvvmToolkit.ModelExtensions;
using WMAApp;
using SimpleMvvmToolkit;
using WMAApp.Helpers;
using WMAApp.Web.Models;
using WMAApp.ViewModels;
using WMAApp.Services.Connection;
using WMAApp.Other;

namespace WMAApp.ViewModels.Connection
{
   /// <summary>
    /// This class contains properties that a View can data bind to.
    /// <para>
    /// Use the <strong>mvvmprop</strong> snippet to add bindable properties to this ViewModel.
    /// </para>
    /// </summary>
    public class DbSelectVM : WMAVMBase<DbSelectVM>
    {
        #region Initialization and Cleanup

        App currApp;
        private ChildWindow popUp;

        // Add a member for IDbSelectSA
        private IDbSelectSA serviceAgent; // Just in case we need to go to db.

        // Default constructor
        public DbSelectVM()
        {
            // Register to get notified when to show myself.
            this.RegisterToReceiveMessages(MessageTokens.sSHOW_DBCONNECTIONS, Show );
            //MessageBus.Default.Register(MessageTokens.sSHOW_DBCONNECTIONS, Show);

            SetPopUpCommand = new DelegateCommand<object>(SetPopUp, CanSetPopUp);
            OkButtonCommand = new DelegateCommand(OkButton, CanOkButton);
            CancelButtonCommand = new DelegateCommand(CancelButton, CanCancelButton);
            ClearButtonCommand = new DelegateCommand(ClearButton, CanClearButton);

            UseAsDefault = false;

            _connections = new ObservableCollection<DbConnection>();

            if (!DesignerProperties.IsInDesignTool)
            {
                currApp = (App)Application.Current;
            }
        }

        /// <summary>
        /// Initializes a new instance of the DbSelectVM class.
        /// </summary>
        /// constructor that accepts IDBSelectSA
        public DbSelectVM(IDbSelectSA _serviceAgent):this()
        {
            // Set service agent
            this.serviceAgent = _serviceAgent;          
        }

        #endregion

        #region Notifications
        // Communication between viewModel and view.

        // Add events to notify the view or obtain data from the view
        public event EventHandler<NotificationEventArgs<Exception>> ErrorNotice;

        // notify view when to show myself
        public event EventHandler<NotificationEventArgs> ShowNotice;

        #endregion

        #region Properties

        private bool _bIsBusy;
        public bool bIsBusy
        {
            get { return _bIsBusy; }
            set
            {
                _bIsBusy = value;
                //SetCanProperties();
                NotifyPropertyChanged(m => m.bIsBusy);
            }
        }

        private ObservableCollection<DbConnection> _connections;
        public ObservableCollection<DbConnection> Connections
        {
            get
            {
                return _connections;
            }

            set
            {
                if (value != _connections)
                {
                    _connections = value;
                    //this.OnPropertyChanged("Connections");
                    NotifyPropertyChanged(m => m.Connections);
                }
            }
        }

        private bool _boUseAsDefault;
        public bool UseAsDefault
        {
            get
            {
                return _boUseAsDefault;
            }
            set
            {
                if (_boUseAsDefault != value)
                {
                    _boUseAsDefault = value;
                    //OnPropertyChanged("UseAsDefault");
                    NotifyPropertyChanged(m => m.UseAsDefault);
                }
            }
        }

        #endregion
        
        // Commands
        #region Commands

        //public DelegateCommand<object> SetPopUpCommand { get; set; }
        private DelegateCommand<object> setPopUpCommand;
        public DelegateCommand<object> SetPopUpCommand
        {
            get
            {
                //if (setPopUpCommand == null) setPopUpCommand =
                //    new DelegateCommand<object>(SetPopUp, CanSetPopUp);
                return setPopUpCommand;
            }
            private set { setPopUpCommand = value; }
        }
        public void SetPopUp(object param)
        {
            popUp = (ChildWindow)param;
            LoadConnectionList();
        }
        private bool CanSetPopUp(object param)
        {
            return true;
        }

        private DelegateCommand okButtonCommand;
        public DelegateCommand OkButtonCommand
        {
            get
            {
                return okButtonCommand;
            }
            private set { okButtonCommand = value; }
        }
        public void OkButton()
        {
            WebContext.Current.User.DefaultConns = "";
            WebContext.Current.User.DbConnected = "";

            foreach (DbConnection dbConn in Connections)
            {
                if (dbConn.boSelected)
                {
                    WebContext.Current.User.DefaultConns += dbConn.sConnString + ";";
                    WebContext.Current.User.DbConnected += dbConn.sName + ";";
                }
            }

            if (WebContext.Current.User.DefaultConns.Length > 0)
            {
                WebContext.Current.User.DefaultConns = WebContext.Current.User.DefaultConns.Substring(0, WebContext.Current.User.DefaultConns.Length - 1);
                WebContext.Current.User.DbConnected = WebContext.Current.User.DbConnected.Substring(0, WebContext.Current.User.DbConnected.Length - 1);
            }

            if (!UseAsDefault)
            {
                WebContext.Current.User.DefaultConns = "";
            }

            WebContext.Current.User.ConnectedToDb = true;
            WebContext.Current.Authentication.SaveUser(true);
            popUp.DialogResult = true;

            // Example SimpleMVVMToolkit messaging:
            
            ////Example 1: Create page value to send saved customer to home page
            //var pageValues = new KeyValuePair<string, object>("SavedCustomer", Customer);
            //// Navigate to home page with selected customer
            //SendMessage(MessageTokens.Navigation, new NotificationEventArgs
            //            <KeyValuePair<string, object>>(PageNames.Home, pageValues));

            // Broadcast message using the MessageBus, specifying a callback that the subscriber can invoke
            //Example 2: SendMessage(MessageTokens.sDbConnectionChanged, 
            //                     new NotificationEventArgs<IncreaseInfo, ApprovalInfo>(null, increaseInfo, OnIncreaseResponse)); 

            // Broadcast notification to other viewmodel listeners that user changed db selection.
            SendMessage(MessageTokens.sDB_CONNECTION_CHANGED,
                        new NotificationEventArgs<ObservableCollection<DbConnection>>(null, Connections));

        }
        private bool CanOkButton()
        {
            return true;
        }

        private DelegateCommand cancelButtonCommand;
        public DelegateCommand CancelButtonCommand
        {
            get
            {
                return cancelButtonCommand;
            }
            private set { cancelButtonCommand = value; }
        }
        public void CancelButton()
        {
            WebContext.Current.User.ConnectedToDb = false;
            WebContext.Current.User.DbConnected = "";

            popUp.DialogResult = false;
        }
        private bool CanCancelButton()
        {
            return true;
        }

        private DelegateCommand clearButtonCommand;
        public DelegateCommand ClearButtonCommand
        {
            get
            {
                return clearButtonCommand;
            }
            private set { clearButtonCommand = value; }
        }
        public void ClearButton()
        {
            // clear the default connection list and then save it per user

            WebContext.Current.User.DefaultConns = "";
            WebContext.Current.Authentication.SaveUser(true);

            UseAsDefault = false;
            // set app setting DefaultConns = null
        }
        private bool CanClearButton()
        {
            return true;
        }

        #endregion


        #region Methods

        public void Show( object sender, NotificationEventArgs e)
        {
            //tell view to show           
            Notify(ShowNotice, new NotificationEventArgs());
          
        }

        // methods that will be called by the view
        public void LoadConnectionList()
        {

            // look up key value pairs from application settings
            int iCount = 0;
            string sValue;
            string[] sConns = null;

            Connections.Clear();

            if (WebContext.Current.User.DefaultConns.Length > 0)
            {
                sConns = WebContext.Current.User.DefaultConns.Split(';');

                UseAsDefault = true;
            }
            else
            {
                UseAsDefault = false;
            }

            if (currApp.appSettings.ContainsKey("NumberOfConnections"))
            {
                iCount = Convert.ToInt32(currApp.appSettings["NumberOfConnections"]);

                for (int i = 1; i <= iCount; i++)
                {
                    DbConnection conn = new DbConnection();

                    if (currApp.appSettings.TryGetValue("Connection" + i.ToString("d2"), out sValue))
                    {
                        conn.sConnString = sValue;
                    }

                    if (currApp.appSettings.TryGetValue("ConnName" + i.ToString("d2"), out sValue))
                    {
                        conn.sName = sValue;
                    }

                    if (UseAsDefault)
                    {
                        foreach (string sConn in sConns)
                        {
                            if (conn.sName == sConn)
                            {
                                conn.boSelected = true;
                            }
                        }
                    }

                    Connections.Add(conn);
                }

            }
        }

       

        // example
        //public void LoadProductData(DateTime _dtStartTime, DateTime _dtEndTime)
        //{
        //    // Load productData for the specified time range
        //    serviceAgent.GetProductDataWStatAndRej
        //        (
        //            _dtStartTime,
        //            _dtEndTime,
        //            (entities, error) => ProductDataLoaded(entities, error)
        //        );

        //    // Reset property
        //    ProductData = null;

        //    // Flip busy flag
        //    IsBusy = true;

        //}
        #endregion

        #region Completion Callbacks

        // example
        //// Add callback methods for async calls to the service agent
        //private void ProductDataLoaded(List<PRODUCT_DATA> _entities, Exception _error)
        //{
        //    // If no error is returned, set the model to entities
        //    if (_error == null)
        //    {
        //        ProductData = new ObservableCollection<PRODUCT_DATA>(_entities);

        //        // STEP 4e5: Turn on dirty-checking after loading products
        //        //serviceAgent.IsDirtyChecking = true;

        //        //string sStatus;
        //        //foreach (PRODUCT_DATA pd in ProductData ) //_WMADBDomainContext.PRODUCT_DATAs)
        //        //{
        //        //    sStatus = "";

        //        //    foreach (PRODUCT_DATA_STAT pds in pd.PRODUCT_DATA_STAT)
        //        //    {
        //        //        sStatus += pds.PRODUCT_DATA_STAT_ID.ToString() + ",";
        //        //    }
        //        //}
        //    }
        //    // Otherwise notify view that there's an error
        //    else
        //    {
        //        NotifyError("Unable to retrieve product data", _error);
        //    }


        //    // We're done
        //    IsBusy = false;
        //}

        #endregion

        #region Helpers

        // Helper method to notify View of an error
        private void NotifyError(string _sMessage, Exception _exError)
        {
            // Notify view of an error
            Notify(ErrorNotice, new NotificationEventArgs<Exception>(_sMessage, _exError));
        }

        #endregion
    }
}
