﻿namespace WMAApp.Views.Connection
{
    using System.ComponentModel;
    using System.Globalization;
    using System.ServiceModel.DomainServices.Client.ApplicationServices;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;
    using WMAApp.Views.Connection.Resources;
    using System;
    using SimpleMvvmToolkit;
    using WMAApp.Other;
    using WMAApp.ViewModels.Connection;

    //todo: maybe some of this logic should be moved into ConnectStatusVM

    /// <summary>
    /// <see cref="UserControl"/> class that shows the current login status and allows login and logout.
    /// </summary>
    public partial class ConnectStatusV : UserControl
    {
       
        /// <summary>
        /// Creates a new <see cref="ConnectStatusV"/> instance.
        /// </summary>
        /// 
        // Reference view model
        ConnectStatusVM vm;

        public ConnectStatusV()
        {
            InitializeComponent();
            // Get vm from data context
            vm = (ConnectStatusVM)DataContext;

            // subscribe to notifications from viewmodel
            vm.DbSelectionChangedNotice += OnDbSelectionChangedNotice;

            if (DesignerProperties.IsInDesignTool)
            {
                VisualStateManager.GoToState(this, "loggedOut", false);
            }
            else
            {
                //???todo: ask joe if he really needs this   this.DataContext = WebContext.Current; //???
                WebContext.Current.Authentication.LoggedIn += this.Authentication_LoggedIn;
                WebContext.Current.Authentication.LoggedOut += this.Authentication_LoggedOut;
                this.UpdateConnectState();               
            }
        }
       
        private void ConnectButton_Click(object sender, RoutedEventArgs e)
        {           
            vm.ShowDbSelections(); 
        }

        private void DisconnectButton_Click(object sender, RoutedEventArgs e)
        {
            // clear db connections
            WebContext.Current.User.ConnectedToDb = false;

            this.UpdateConnectState();
        }

        void OnDbSelectionChangedNotice(object sender, NotificationEventArgs e)
        {
            this.UpdateConnectState();
        }
        public void DbSelectClosed(object sender, EventArgs e) //todo: is this needed now
        {
            this.UpdateConnectState();
        }

        private void Authentication_LoggedIn(object sender, AuthenticationEventArgs e)
        {
            this.UpdateConnectState();
        }

        private void Authentication_LoggedOut(object sender, AuthenticationEventArgs e)
        {
            this.UpdateConnectState();
        }

        private void UpdateConnectState()
        {            
            if (WebContext.Current.User.IsAuthenticated)
            {
                this.connectedText.Text = string.Format(
                                    CultureInfo.CurrentUICulture,
                                    ViewStrings.DbConnected,
                                    WebContext.Current.User.DbConnected);
                
            }
            else
            {
                this.connectedText.Text = ViewStrings.AuthenticatingMessage;
            }

            if (WebContext.Current.Authentication is WindowsAuthentication)
            {
                VisualStateManager.GoToState(this, "connectAuth", true);
            }
            else
            {
                if (WebContext.Current.User.IsAuthenticated)
                {
                    if (WebContext.Current.User.ConnectedToDb)
                    {
                        VisualStateManager.GoToState(this, "dbConnected", true);
                    }
                    else
                    {
                        VisualStateManager.GoToState(this, "loggedIn", true);
                    }
                }
                else
                {
                    VisualStateManager.GoToState(this, "loggedOut", true);
                }
            }
        }
    }
    
}
