﻿#define LOAD_PDB_DEFNS_BEFORE_DELETION // todo: necessary until we figure out better deletion approach ( like composition )
#define USE_GRID_FOR_DD
#define SUPPORT_UNDO_REDO
#define SUPPORT_TEST_LIST_RESIZE
#define SUPPORT_TEST_LIST_RESIZE_ON_INIT_ROW // WARNING: This approach becomes very time costly when you have a large number of rows that are being iterated...
#define SUPPORT_TEST_POINT_LIST_RESIZE
#define SUPPORT_FULL_REP_RESIZE
#define SUPPORT_CHART_LAUNCH_FROM_FUL_REP
#define SUPPORT_TEST_LIST_BANDS
#define GET_ACTUAL_PIVOT_DATA
#define SUPPORT_PERSISTENCE
#if SUPPORT_PERSISTENCE
#define USE_GROUP_PERSISTENCE
#define XDM_NAVIGATE_BY_INDEX
#endif

namespace WMAApp.Views.Home
{
    #region Using
    using System.Windows.Controls;
    using System.Windows.Navigation;
    using System.ServiceModel.DomainServices.Client.ApplicationServices;
    using SimpleMvvmToolkit;
    using WMAApp.ViewModels.Home;
    using System;
    using System.Windows;
    using System.Diagnostics;
    using WMAApp.Helpers;
    using Infragistics.Controls.Editors;
    using Infragistics;
    using System.Collections.Generic;
    using System.Collections;
    using Infragistics.Controls.Menus;
    using Infragistics.Controls.Editors.Primitives;
    using Infragistics.Documents.Excel;
    using WMAApp.Web.Models;
    using System.Windows.Data;
    using System.Globalization;
    using System.Collections.ObjectModel;
    using WMAApp.Other;
    using Infragistics.Controls.Grids;
    using System.IO;
    using Infragistics.Controls.Layouts;
    using System.Windows.Media.Imaging;
    using Infragistics.Controls.Charts;
    using System.Linq;

#if SUPPORT_UNDO_REDO
    using Infragistics.Undo;
#endif
    using System.Windows.Threading;
    using System.Threading;
    using WMAApp.WMASharedClasses;
    using WMAApp.Controls;
    using System.Windows.Markup;
    using WMAApp.Views.Home.Resources;
    using System.Windows.Media;
    using System.Text;
    using Infragistics.Controls;
    using Infragistics.Imaging;
    using Infragistics.Controls.Grids.Primitives;
    using Infragistics.Olap.FlatData;
    using Infragistics.Olap;
    using Infragistics.Olap.Data;
    using System.Linq.Expressions;
    using Infragistics.Persistence;
    using System.IO.IsolatedStorage;
    using System.Windows.Controls.Primitives;
    using WMAApp.Views.Charts;
    #endregion Using

    /// <summary>
    /// Home page for the application.
    /// </summary>
    public partial class HomeV : Page
    {
        #region Constructors
        /// <summary>
        /// Creates a new <see cref="HomeV"/> instance.
        /// </summary>
        public HomeV()
        {
            InitializeComponent();

            Title = ApplicationStrings.HomePageTitle;

            m_vm = (HomeVM)this.DataContext;

            // Subscribe to notifications from the view model
            m_vm.evErrorNotice += OnErrorNotice;
            m_vm.evOverwritePDBNotice += OnOverwritePDBNotice;
            m_vm.evGeneralAcknowledgmentNotice += OnGeneralAcknowledgmentNotice;
            // notify me to setupbindings.
            m_vm.evSetupBindings += vSetupBindings;
            // notify me if I need to reset any filters, sorting, etc.
            m_vm.evResetTestListTweaks += vResetTestListTweaks;
            // notify me when I need to contruct visible tests list
            m_vm.evBuildVisibleTestList += vBuildVisibleTestList;
            // Notify me when I need to show a full report.
            m_vm.evShowFullRep += vShowFullRepEx;
            m_vm.evShowTPData += vShowTPDataEx;
            m_vm.evShowTPLimitData += vShowTpLimitsEx;
            m_vm.evShowTPPivotGridData += vShowTPPivotGridDataEx;

            m_WMAMethods = new WMAMethods();
            m_bNewPDB = false;
            //m_sPDBName = System.String.Empty;
            //iInitializeRowCount = 0;
            iBatchNdx = 0;
            iBatchCount = 0;
            sContextMenuXamGrid = string.Empty;
            dScrollBarWidth = 45; // default to 45 but will be updated programatically
            #if SUPPORT_PERSISTENCE
                m_bTestListLoaded = false;
                m_persistenceSettings = new PersistenceSettings();
                m_persistenceSettingsDM = new PersistenceSettings();
#if USE_GROUP_PERSISTENCE
                ////////m_persistenceGroup = new PersistenceGroup();
#endif
            #endif

            //default data selection settings
            radCurrent.IsChecked = m_vm.boByCurrent;
            radHour.IsChecked = m_vm.boHour;

            // default data refresh timer
            iTimerInterval = 120;
            tmrDataRefresh = new DispatcherTimer();
            tmrDataRefresh.Interval = new TimeSpan(0,0,iTimerInterval);
            tmrDataRefresh.Tick += new EventHandler(tmrDataRefresh_Tick);
        }


        #endregion // Constructors

        #region Members

        //const int iIG_DEF_SCROLLBAR_WIDTH = 17 + 2; // I add 2 so that it's edge is not cutoff.
        const int iCONTENT_PANE_LIMIT = 20; // todo: move to appropriate place later.
        private HomeVM m_vm;
        //private string m_sPDBName;
        private WMAMethods m_WMAMethods;
        private bool m_bNewPDB;
        private Workbook m_Workbook;
        private Stream m_stream;
        //private int iInitializeRowCount = 0;

        private int iBatchNdx;
        private int iBatchCount;
        private string sContextMenuXamGrid;
        private double dScrollBarWidth;

#if SUPPORT_PERSISTENCE
        bool m_bTestListLoaded;
        private PersistenceSettings m_persistenceSettings;
        private PersistenceSettings m_persistenceSettingsDM;
        private MemoryStream m_ms;
        private MemoryStream m_msDM;
        #if USE_GROUP_PERSISTENCE
            private PersistenceGroup m_persistenceGroup;
        #endif

        #if XDM_NAVIGATE_BY_INDEX
            TabGroupPane mytgpList;
            ContentPane mycpTestList;
            TabGroupPane mytgpData;
        #endif
#endif

        DispatcherTimer tmrDataRefresh;

        int iTimerInterval;

        #endregion // Members

        #region Methods

        private void SetControlsVisible(System.Windows.Visibility _visState)
        {
            expFilters.Visibility = _visState;
            expPdbSettings.Visibility = _visState;
            grDataArea.Visibility = _visState;
        }

        #region Navigation

        /// <summary>
        /// Executes when the user navigates to this page.
        /// </summary>
        protected override void OnNavigatedTo(NavigationEventArgs _evArgs)
        {
            // Set initial state of controls, when the page is navigated to based on login

            if (WebContext.Current.User.IsAuthenticated)
                SetControlsVisible(System.Windows.Visibility.Visible);
            else
                SetControlsVisible(System.Windows.Visibility.Collapsed);

            // wired up the routines to deal with user logging in and out of application

            WebContext.Current.Authentication.LoggedIn += new System.EventHandler<AuthenticationEventArgs>(Authentication_LoggedIn);
            WebContext.Current.Authentication.LoggedOut += new System.EventHandler<AuthenticationEventArgs>(Authentication_LoggedOut);
        }
        #endregion
        #region LogInOut
        void Authentication_LoggedIn(object _sender, AuthenticationEventArgs _evArgs)
        {
            m_vm.vGetDataSourceSelection(WebContext.Current.User.DefaultConns);
            SetControlsVisible(System.Windows.Visibility.Visible);
        }
        void Authentication_LoggedOut(object _sender, AuthenticationEventArgs _evArgs)
        {
            SetControlsVisible(System.Windows.Visibility.Collapsed);
        }
        #endregion
        #region ErrorNotification

        void OnErrorNotice(object _sender, NotificationEventArgs<Exception> _evArgs)
        {
            // Show user message string
            if (_evArgs.Data != null)
                MessageBox.Show(_evArgs.Message + ":" + _evArgs.Data.Message, "Error", MessageBoxButton.OK);
            else
                MessageBox.Show(_evArgs.Message, "Error", MessageBoxButton.OK);

            // Trace information
            //if (_evArgs.Data != null)
            //{
            //    Debug.WriteLine(_evArgs.Data.ToString());
            //}
        }
        #endregion
        #region Filtration

        void popup_Opened(object sender, EventArgs e)
        {
            Popup popup = sender as Popup;
            DependencyObject itemsPanel = FindFirstRow(popup.Child);
            if (itemsPanel != null && VisualTreeHelper.GetChildrenCount(itemsPanel) > 0)
            {
                double width = 0;
                ComboCellsPanel cellsPanel = VisualTreeHelper.GetChild(itemsPanel, 1) as ComboCellsPanel;
                for (int i = 0; i < VisualTreeHelper.GetChildrenCount(cellsPanel); i++)
                {
                    switch (VisualTreeHelper.GetChild(cellsPanel, i).GetType().ToString())
                    {
                        case "Infragistics.Controls.Editors.ComboCellControl":
                            ComboCellControl control = VisualTreeHelper.GetChild(cellsPanel, i) as ComboCellControl;
                            if (control != null)
                                width += control.Cell.Column.ActualWidth;
                            break;

                        case "Infragistics.Controls.Editors.RowSelectionCheckBoxCellControl":

                            RowSelectionCheckBoxCellControl control1 = VisualTreeHelper.GetChild(cellsPanel, i) as RowSelectionCheckBoxCellControl;
                            if (control1 != null)
                                width += control1.Cell.Column.ActualWidth;
                            break;
                    }
                }

                if (width > ((popup.Child as System.Windows.Controls.Grid).Children[0] as Border).Width)
                    ((popup.Child as System.Windows.Controls.Grid).Children[0] as Border).Width = width + 3;
            }
        }

        /// <summary>
        /// Helper method used to find the first grid row in the visual tree.
        /// </summary>
        /// <param name="root"></param>
        /// <returns></returns>
        private DependencyObject FindFirstRow(DependencyObject root)
        {
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(root); )//i++)
            {
                DependencyObject dObj = VisualTreeHelper.GetChild(root, i);
                if (dObj.GetType() == typeof(MultiColumnComboItemsPanel))//typeof(ComboCellsPanel))
                    return dObj;

                dObj = FindFirstRow(dObj);
                    return dObj;
            }

            return null;
        }

        private void vSetupComboList(object _sender)
        {
            Grid grid = VisualTreeHelper.GetChild(_sender as XamMultiColumnComboEditor, 0) as Grid;
            Popup popup = ((grid.Children[0] as Border).Child as Grid).Children[0] as Popup;

            // This will change the white space to gray, you really can't get rid of the space though unless the dropdown size
            // is restricted.
            ((popup.Child as Grid).Children[0] as Border).Background = new SolidColorBrush(Colors.Gray);

            // Handle the popup opened event.  The grid rows will exist by the time this event is fired.
            popup.Opened += new EventHandler(popup_Opened);
        }

        private void cmbFamily_Loaded(object _sender, EventArgs _evArgs)
        {
            vSetupComboList(_sender);
        }
        
        private void cmbStand_Loaded(object _sender, EventArgs _evArgs)
        {
            vSetupComboList(_sender);
        }

        private void cmbModels_Loaded(object _sender, EventArgs _evArgs)
        {
            vSetupComboList(_sender);
        }

        private void cmbFamily_DropDownClosed(object _sender, EventArgs _evArgs)
        {
            MessageBus.Default.Notify(MessageTokens.sSHOW_OKCANCELMSGBOX, this, new NotificationEventArgs());

            m_vm.OnChangeFamilySel();
        }

        private void cmbModels_DropDownClosed(object _sender, EventArgs _evArgs)
        {
            m_vm.OnChangeModelsSel(cmbModels);
        }

        private void cmbStand_DropDownClosed(object _sender, EventArgs _evArgs)
        {
            m_vm.OnChangeStandSel(cmbStand);
        }

        #endregion
        #region Page
        #region Events
        private void Page_Loaded(object _sender, RoutedEventArgs _e)
        {
            //// determine the width of a default IG vertical scroll bar.
            //ContentPane cp = dmPDB.FindPane("cpTestList") as ContentPane;
            //IProvideScrollInfo scrollInfo = cp.FindName("xgTestList") as IProvideScrollInfo;
            //double dScrollBarWidth = scrollInfo.VerticalScrollBar.ActualWidth + 2;

            //this.xdcChart.WindowRectChanged += OnDataChartWindowRectChanged;
            //this.sldVerticalZoom.ValueChanged += OnVerticalZoomValueChanged;
            //this.sldHorizontalZoom.ValueChanged += OnHorizontalZoomValueChanged;
           
            //xmDataChart.DragModifier = ModifierKeys.Shift;
#if SUPPORT_PERSISTENCE
#if USE_GROUP_PERSISTENCE
            m_persistenceGroup = (PersistenceGroup)this.Resources["pgTestList"];
            ////vRestoreLayoutFromIsolatedStorage(m_persistenceGroup);
#else
            vRestoreLayoutFromIsolatedStorage(m_persistenceSettings, sender as XamGrid);
#endif
#endif
        }
        #endregion Events
        #endregion Page

        #region PDB

        #region DockManager

        #region Data members
        // Default to true for startup.
        bool _bTestListStillCalculating = true; // when goes to false this means all the initial rows have been created / initialized 
        bool _bTPListStillCalculating = true;
        bool _bFullRepStillCalculating = true;

        // The pane is not initially ready.
        bool _bTestListPaneReady = false;
        bool _bTPListPaneReady = false;
        bool _bFullRepPaneReady = false;
        #endregion Data members

        #region Events
        private void evhtgbList_Loaded(object _sender, RoutedEventArgs e)
        {
            // Set the active pane for the tab group.
            Infragistics.Controls.Layouts.TabGroupPane tgp = (Infragistics.Controls.Layouts.TabGroupPane)_sender;
            if ( tgp.Panes.Count > 0 )
                tgp.Panes[0].IsActivePane = true;
            //tgp.Panes[0].Width = 1000; // this will be trimmed down in xgTestList_InitializeRow

        }
        private void evhTestListContentPane_SizeChanged(object _sender, SizeChangedEventArgs _evArgs)
        {
#if SUPPORT_TEST_LIST_RESIZE
            // As long as we're no longer calculating.
            if (!_bTestListStillCalculating)
            {                                
                // If we get here that means all the initial rows have been created.
                _bTestListPaneReady = true;

                XamGrid xg = (_sender as ContentPane).FindName("xgTestList") as XamGrid;

                if (xg != null)
                {
                    // calculate widths with percentages so they keep the same relative size to each other.
#if SUPPORT_TEST_LIST_BANDS                    
                    foreach (object o in xg.Columns)
                    {                        
                        if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                        {
                            Column col = o as Column;
                            // if column was collapsed, ignore it
                            if (col.Visibility == System.Windows.Visibility.Visible)
                            {
                                if (col.Tag != null)
                                {
                                    ColumnInfo cInfo = col.Tag as ColumnInfo;
                                    int pixels = (int)(cInfo.dRelativeWidth * (_evArgs.NewSize.Width - dScrollBarWidth));
                                    col.Width = new ColumnWidth(pixels, false);
                                }
                            }
                        }
                    }
#else
                    foreach (Column col in xg.Columns)
                    {
                        if (col.Tag != null) // ask rob
                        {
                            ColumnInfo cInfo = col.Tag as ColumnInfo;
                            int pixels = (int)(cInfo.dRelativeWidth * (_evArgs.NewSize.Width - dScrollBarWidth));
                            col.Width = new ColumnWidth(pixels, false);
                        }
                    }
#endif
                }
            }
#else
            ContentPane contentPane = (ContentPane)_sender;
            XamGrid xg = (XamGrid)contentPane.FindName("xgTestList");
            if (xg != null) xg.Width = _evArgs.NewSize.Width - dScrollBarWidth;
#endif

        }
        private void evhTPListContentPane_SizeChanged(object _sender, SizeChangedEventArgs _evArgs)
        {
#if SUPPORT_TEST_POINT_LIST_RESIZE
            // As long as we're no longer calculating.
            if (!_bTPListStillCalculating)
            {                                
                // If we get here that means all the initial rows have been created.
                _bTPListPaneReady = true;

                XamGrid xg = (_sender as ContentPane).FindName("xgTPList") as XamGrid;
                if (xg != null)
                {
#if SUPPORT_TEST_LIST_BANDS                    
                    foreach (object o in xg.Columns)
                    {
                        if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                        {
                            Column col = o as Column;
                            // if column was collapsed, ignore it
                            if (col.Visibility == System.Windows.Visibility.Visible)
                            {
                                if (col.Tag != null) // ask rob
                                {
                                    ColumnInfo cInfo = col.Tag as ColumnInfo;
                                    int pixels = (int)(cInfo.dRelativeWidth * (_evArgs.NewSize.Width - dScrollBarWidth));
                                    col.Width = new ColumnWidth(pixels, false);
                                }
                            }
                        }
                    }
#else
                    // calculate widths with percentages so they keep the same relative size to each other.
                    foreach (Column col in xg.Columns)
                    {
                        if (col.Tag != null) // ask rob
                        {
                            ColumnInfo cInfo = col.Tag as ColumnInfo;
                            int pixels = (int)(cInfo.dRelativeWidth * (_evArgs.NewSize.Width - dScrollBarWidth));
                            col.Width = new ColumnWidth(pixels, false);
                        }
                    }
#endif
                }
            }
#else
            ContentPane contentPane = (ContentPane)_sender;
            XamGrid xg = (XamGrid)contentPane.FindName("xgTPList");
            if (xg != null) xg.Width = _evArgs.NewSize.Width - dScrollBarWidth;
#endif

        }        
        private void evhdmPDB_PaneClosing(object _sender, CancellablePaneEventArgs _evArgs)
        {
            // trap this event to prevent the dockmanager from inserting this closed pane into the closedPane collection ( which I cannot remove from).
            XamDockManager xdm = (XamDockManager)_sender;

            if (_evArgs.Pane.Name == "tgpData")
            {
                _evArgs.Cancel = true;
            }
            else if (_evArgs.Pane is ContentPaneEx)
            {
                TabGroupPane tgp = _evArgs.Pane.OwnerPane as TabGroupPane;                
                // when I hit the x close button on the tgpData pane in the upper right, it closes the pane that is active but then does not switch you to a valid tab so
                // make first pane in collection be the active one
                if (tgp != null ) {
                    if (tgp.Panes.Count > 2)
                    {
                        // remember that the first two panes are dummy panes
                        foreach (ContentPaneEx cpex in tgp.Panes)
                        {
                            if (_evArgs.Pane.Name != cpex.Name && !cpex.Name.StartsWith("cpDummy"))
                            {
                                cpex.IsActivePane = true;
                                break;
                            }
                        }
                    }
                }
            }
        }

        private void evhdmPDB_PaneClosed(object _sender, PaneEventArgs _evArgs)
        {
 /*           // if there are no more open panes in tgpData ( except the dummy panes per IG:92858 ), make tgpData "small" again
#if XDM_NAVIGATE_BY_INDEX
            Infragistics.Controls.Layouts.TabGroupPane tgpData = mytgpData;
#else
                        Infragistics.Controls.Layouts.TabGroupPane tgpData = (Infragistics.Controls.Layouts.TabGroupPane)dmPDB.FindPane("tgpData");
#endif
            if (tgpData != null)
            {
                if (tgpData.Panes.Count == 2)
                {
                    tgpData.SplitWeight = 0;
                }
            }*/
        }
        private void evhdmPDB_Loaded(object sender, RoutedEventArgs e)
        {
            vRestoreLayoutFromIsolatedStorageDM(m_persistenceSettingsDM, sender as XamDockManager);
#if XDM_NAVIGATE_BY_INDEX
            vBuildXDMData();
#endif

        }
        public class ColumnInfo
        {
            public double dRelativeWidth { get; set; }
            public double dWidth { get; set; }
        }
                
        #endregion Events

#if SUPPORT_PERSISTENCE
        #region Persistence
        private const string sdmPDBIsfName = "dmPDBLayout";
        private void xmiDMPersistLayout_Click(object sender, EventArgs _evArgs)
        {
                try
                {
                    using (IsolatedStorageFile isfStore = IsolatedStorageFile.GetUserStoreForApplication())
                    {
                        /*
                        // Create directory to save the isf.
                        isfStore.CreateDirectory("WMAApp");
                        string sSubDirectory = Path.Combine("WMAApp", "Layouts");
                        isfStore.CreateDirectory(sSubDirectory);

                        // Create a file in the subdirectory.
                        IsolatedStorageFileStream subDirFile =
                            isfStore.CreateFile(Path.Combine(sSubDirectory, sdmPDBIsfName));
                        subDirFile.Close();
                        */
                        //string filePath = Path.Combine(sSubDirectory, sIsfName);
                        if (isfStore.FileExists(sdmPDBIsfName))
                            isfStore.DeleteFile(sdmPDBIsfName);

                        using (MemoryStream m_msDM = PersistenceManager.Save(dmPDB, m_persistenceSettingsDM ))
                        {
                            // will the data fit in the available space?
                            if ((int)m_msDM.Length > isfStore.AvailableFreeSpace)
                            {
                                // Request more quota space.
                                vIncreaseStoreQuota((int)m_msDM.Length - isfStore.AvailableFreeSpace + 1000);
                            }

                            using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(sdmPDBIsfName, FileMode.OpenOrCreate, isfStore))
                            {
                                // Write file in the isolated storage
                                stream.Write(m_msDM.ToArray(), 0, (int)m_msDM.Length);
                            }
                        }
                    }
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.Message);
                    //throw; //todo
                }
        }
        private void xmiDMRestoreLayout_Click(object sender, EventArgs _evArgs)
        {
            vRestoreLayoutFromIsolatedStorageDM(m_persistenceSettingsDM, dmPDB);
        }
        #region Store Methods
       
        private void vRestoreLayoutFromIsolatedStorageDM(PersistenceSettings _oSettings, object _oControl) {
            try
            {
                using (IsolatedStorageFile isfStore = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    if (isfStore.FileExists(sdmPDBIsfName))
                    {
                        using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(sdmPDBIsfName, FileMode.OpenOrCreate, isfStore))
                        {
                            try
                            {
                                PersistenceManager.Load(_oControl as XamDockManager, stream, _oSettings);
                            }
                            catch (Exception)
                            {
                                //throw; //todo
                            }

                        }
                    }
                }
            }
            catch (IsolatedStorageException e)
            {
                MessageBox.Show(e.Message);
                //throw;
            }
        }

        #endregion Store Methods

#if XDM_NAVIGATE_BY_INDEX
        private void vBuildXDMData()
        {
            //myspPDB = dmPDB.Panes[0] as SplitPane;
            //mytgpList = myspPDB.Panes[0] as TabGroupPane;
            mytgpList = dmPDB.Panes[0] as TabGroupPane;
            mycpTestList = mytgpList.Panes[0] as ContentPane;

            //myspPDBData = myspPDB.Panes[1] as SplitPane;
            //mytgpData = myspPDBData.Panes[0] as TabGroupPane;
            //mytgpCharts = myspPDBData.Panes[1] as TabGroupPane;
            //mytgpPivotGrid = myspPDBData.Panes[2] as TabGroupPane;

            DocumentContentHost dchPanes = dmPDB.Content as DocumentContentHost;
            mytgpData = dchPanes.Panes[0] as TabGroupPane;
            //mytgpCharts = dchPanes.Panes[1] as TabGroupPane;
            //mytgpPivotGrid = dchPanes.Panes[0] as TabGroupPane;
        }

#endif

        #endregion Persistence
#endif
        #endregion DockManager

        #region Tests
        #region Event handlers
        private void evhxgTestList_Loaded(object sender, RoutedEventArgs e)
        {
           
#if SUPPORT_PERSISTENCE
            // This loaded method fires even when we navigate from another tab back to this one.
            // Only process the persistence on first hit.
            if (!m_bTestListLoaded)
            {
                m_bTestListLoaded = true;
#if USE_GROUP_PERSISTENCE
                SaveInitialTestListSettings();
                ////m_persistenceGroup = (PersistenceGroup)this.Resources["pgTestList"];
                vRestoreLayoutFromIsolatedStorage(m_persistenceGroup);
#else
            vRestoreLayoutFromIsolatedStorage(m_persistenceSettings, sender as XamGrid);
#endif
            }
#endif
        }
        private void evhxgTestList_InitializeRow(object _sender, InitializeRowEventArgs _evArgs)
        {            
        #if SUPPORT_TEST_LIST_RESIZE_ON_INIT_ROW
        // WARNING: This approach becomes very time costly when you have a large number of rows that are being iterated...

            // This check keeps the pane from performing these operations when a new row is added due
            // to virtualization.
            if (!_bTestListPaneReady)
            {               
                // It's ok to do this every frame since each subsequent row can only increase the column width.
                double totalWidth = 0;
#if SUPPORT_TEST_LIST_BANDS
                // Calculate the total width taken up by all the columns.
                foreach (object o in (_sender as XamGrid).Columns)
                    if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                    {
                        totalWidth += (o as Column).ActualWidth;
                    }

                // Now calculate the percentages for each of the columns.
                foreach (object o in (_sender as XamGrid).Columns)
                {
                    if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                    {
                        Column col = o as Column;
                        ColumnInfo cInfo = new ColumnInfo();
                        cInfo.dRelativeWidth = col.ActualWidth / totalWidth;
                        cInfo.dWidth = col.ActualWidth;

                        // Store it for later use.
                        col.Tag = cInfo;
                    }
                }
#else
                // Calculate the total width taken up by all the columns.
                foreach (Column col in (_sender as XamGrid).Columns)
                    totalWidth += col.ActualWidth;

                // Now calculate the percentages for each of the columns.
                foreach (Column col in (_sender as XamGrid).Columns)
                {
                    ColumnInfo cInfo = new ColumnInfo();
                    cInfo.dRelativeWidth = col.ActualWidth / totalWidth;
                    cInfo.dWidth = col.ActualWidth;

                    // Store it for later use.
                    col.Tag = cInfo;
                }
#endif

                // Use a dispatcher to delay setting the panes width.
                Dispatcher.BeginInvoke(() =>
                {
                    // determine the width of a default IG vertical scroll bar.
                    //if (dScrollBarWidth <= 0)
                    //{
#if XDM_NAVIGATE_BY_INDEX
                    ContentPane cp = mycpTestList;
#else
                                        ContentPane cp = dmPDB.FindPane("cpTestList") as ContentPane;
#endif
                    if (cp != null)
                    {
                        IProvideScrollInfo scrollInfo = cp.FindName("xgTestList") as IProvideScrollInfo;
                        dScrollBarWidth = scrollInfo.VerticalScrollBar.ActualWidth + 20;
                        //}

                        // Add some extra padding.
                        cp.FloatWidth = totalWidth + dScrollBarWidth;

                    }
                    // The order in which things occur is that this dispatcher code will be executed a number of times for each row initialized.
                    // This will also set calculating to false.  Once these initial rows are finished, the SizeChanged event on the pane will
                    // be executed.
                    _bTestListStillCalculating = false;
                });
            }
#endif
        }
        private void evhxgTestList_CellDoubleClicked(object _sender, CellClickedEventArgs _evArgs)
        {      
#if SUPPORT_UNDO_REDO                    
            if (_evArgs.Cell.Row.Data.GetType() != typeof(ProductDataUR)) return;
#else
            
            if ( _evArgs.Cell.Row.Data.GetType() != typeof(ProductData) ) return;
#endif


            iBatchNdx = iBatchCount = 0; // we are not doing batch style
            // get the test results for this test.

#if SUPPORT_UNDO_REDO
            ProductDataUR pd = _evArgs.Cell.Row.Data as ProductDataUR;
#else
            ProductData pd = _evArgs.Cell.Row.Data as ProductData;
#endif

            m_vm.vGetTestResults(new TestDescriptor(pd.sFamilyId, pd.dtTime, pd.dtTime.ToString(), pd.sStand, pd.sModel, pd.sSerial));
        }
        private void evhxgTestList_MouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {   // IG:95417 Make context menu visibible when right click on grid occurs until IG bug is fixed. 
            Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(sender as XamGrid);
            cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Visible;
        }

        #endregion Event handlers
        #region Filtration

        private void evhxgTestList_Filtered(object _sender, Infragistics.Controls.Grids.FilteredEventArgs _evArgs)
        {
#if DO_ONCE_WHEN_NEEDED
            Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
            vBuildVisibleTestList(xg);  
#endif

        }

        #endregion //Filtration
        #region Deletion
        // Handled completely by control 
        //private void xmiDeleteSelectedTests(object _sender, EventArgs _evArgs)
        //{
        //    //vm.DeletedSelectedTests();
        //}
        private void evhxgTestList_RowDeleted(object _sender, Infragistics.Controls.Grids.RowEventArgs _evArgs)
        {
#if DO_ONCE_WHEN_NEEDED
            Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
            vBuildVisibleTestList(xg);  
#endif

        }
        private void xmiDeletePDB_Click(object _sender, EventArgs _evArgs)
        {
            xdwDeletePDBs.WindowState = Infragistics.Controls.Interactions.WindowState.Normal;
        }
        private void evhbtnDeletePDBs_OK_Click(object _sender, RoutedEventArgs _evArgs)
        {
            // Verify that user chose a pdb.
#if USE_GRID_FOR_DD
            if (xgDeletePDBNames.SelectionSettings.SelectedRows.Count > 0)
#else
            if (xceDeletePDBNames.SelectedItems.Count > 0)
#endif

            {
                xdwDeletePDBs.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
                // delete the selected pdbdefn(s)
                // populate a collection of the selected pdb defn sakeys ( to be used by the viewmodel )

                PDB_DEFN pdb;
                m_vm.lstSelectedPDBDefnsForDeletion.Clear();
#if USE_GRID_FOR_DD
                foreach (Infragistics.Controls.Grids.Row Item in xgDeletePDBNames.SelectionSettings.SelectedRows)
                {
                    pdb = (PDB_DEFN)Item.Data;
                    // Disallow deletion of the "active" pdb
                    if (txtActivePDBName.Text != pdb.NAME)
                    {
                        m_vm.lstSelectedPDBDefnsForDeletion.Add(pdb);
                    }
                }
#else
                foreach (PDB_DEFN Item in xceDeletePDBNames.SelectedItems)
                                    {
                    // Disallow deletion of the "active" pdb
                    if (txtActivePDBName.Text != Item.NAME)
                    {
                        pdb = (PDB_DEFN)Item;
                        m_vm.lstSelectedPDBDefnsForDeletion.Add(pdb);
                    }
                }
#endif

#if LOAD_PDB_DEFNS_BEFORE_DELETION
                // use approach of loading pdb's and then deleting them
                m_vm.vDeleteSelectedPDBs_Load();
#else
                m_vm.vDeleteSelectedPDBs_NoLoad();// doesn't work consistently
#endif
#if USE_GRID_FOR_DD
                xgDeletePDBNames.SelectionSettings.SelectedRows.Clear();
#else
                // clear out the editor box
                xceDeletePDBNames.SelectedItem = null; // check
#endif
            }
            else
            {
                var mbResult = MessageBox.Show("You must select at least one personal database", "Confirmation", MessageBoxButton.OK);
            }

        }
        private void evhbtnDeletePDBs_Cancel_Click(object _sender, RoutedEventArgs _evArgs)
        {
            xdwDeletePDBs.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
        }
        private void evhxdwDeletePDB_WindowStateChanging(object _sender, Infragistics.Controls.Interactions.WindowStateChangingEventArgs _evArgs)
        {
            switch (_evArgs.NewWindowState)
            {
                case Infragistics.Controls.Interactions.WindowState.Normal:
                    m_vm.vSortPDBDefns();
                    break;
            }
        }
        
#if SUPPORT_UNDO_REDO
        private void evhxgTestList_RowAdding(object _sender, Infragistics.Controls.Grids.CancellableRowAddingEventArgs _evArgs)
        { /* adding not allowed
            string description = "";
            string detailedDescription = "Add New Row";

            // Group several undo units into one item
            UndoTransaction transaction = this._undoManager.StartTransaction(description, detailedDescription);
            new DispatcherSynchronizationContext().Post(new SendOrPostCallback(vCommitTransaction), transaction); */
        }
        private void evhxgTestList_RowDeleting(object _sender, CancellableRowEventArgs _evArgs)
        {
            //todo: how handle multiple rows????
            var oData = ((WMAApp.Helpers.ProductDataUR)(((Infragistics.Controls.Grids.RowBase)(_evArgs.Row)).Data));
            string sRowInfo = oData.sModel + " " + oData.sSerial + " " + oData.sStand + " " + oData.dtTime.ToString();
            string description = "";
            string detailedDescription = "Remove Row:" + sRowInfo; 

            // Group several undo units into one item
            UndoTransaction transaction = this._undoManager.StartTransaction(description, detailedDescription);
            new DispatcherSynchronizationContext().Post(new SendOrPostCallback(vCommitTransaction), transaction);
        }
        
        private void vCommitTransaction(object _utObj)
        {
            if (_utObj != null)
            {
                UndoTransaction transaction = _utObj as UndoTransaction;
                if (!transaction.IsClosed)
                {
                    transaction.Commit();
                }
            }
        }
        private UndoManager _undoManager
        {
            //error Unable to cast object of type 'WMAApp.ViewModels.Home.HomeVM' to type 'WMAApp.ViewModels.Home.ProductDataURVM'.
            //get { return ((ProductDataURVM)this.DataContext).UndoManager; }
            get { return ((HomeVM)this.DataContext).oProductDataURVM.UndoManager; }
        }
#else
        private void evhxgTestList_RowDeleting(object _sender, CancellableRowEventArgs _evArgs)
        { }
#endif

        #endregion //Deletion
        #region Retrieve Part Tests
        private void xmiRetrievePartTests_Click(object _sender, EventArgs _evArgs)
        {
            m_vm.vRetreivePartTests();

        }
        #endregion //Retrieve Part Tests]
        #if SUPPORT_PERSISTENCE
        #region Persistence
        private const string sIsfName = "xgTestListLayout";
        private void xmiGridPersistLayout_Click(object sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                try
                {
                    using (IsolatedStorageFile isfStore = IsolatedStorageFile.GetUserStoreForApplication())
                    {
                        /*
                        // Create directory to save the isf.
                        isfStore.CreateDirectory("WMAApp");
                        string sSubDirectory = Path.Combine("WMAApp", "Layouts");
                        isfStore.CreateDirectory(sSubDirectory);

                        // Create a file in the subdirectory.
                        IsolatedStorageFileStream subDirFile =
                            isfStore.CreateFile(Path.Combine(sSubDirectory, sIsfName));
                        subDirFile.Close();
                        */
                        //string filePath = Path.Combine(sSubDirectory, sIsfName);
                        if (isfStore.FileExists(sIsfName))
                            isfStore.DeleteFile(sIsfName);
                        // Override or augment persistence settings if I want to here...
                        //PropertyNamePersistenceInfo prop = new PropertyNamePersistenceInfo();
                        //prop.PropertyName = "GroupBySettings";
                        //m_persistenceSettings.PropertySettings.Add(prop);
                        
#if USE_GROUP_PERSISTENCE
                        using (MemoryStream m_ms = PersistenceManager.Save(m_persistenceGroup))
                        {
                            //MemoryStream memoryStream = PersistenceManager.Save(m_persistenceGroup);
#else
                        using (MemoryStream m_ms = PersistenceManager.Save(xg /*, m_persistenceSettings */))
                        {
                            //MemoryStream memoryStream = PersistenceManager.Save(xg); //???? , m_persistenceSettings);
#endif
                            // will the data fit in the available space?
                            if ((int)m_ms.Length > isfStore.AvailableFreeSpace)
                            {
                                // Request more quota space.
                                vIncreaseStoreQuota((int)m_ms.Length - isfStore.AvailableFreeSpace + 1000);
                            }

                            using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(sIsfName, FileMode.OpenOrCreate, isfStore))
                            {
                                // Write file in the isolated storage
                                stream.Write(m_ms.ToArray(), 0, (int)m_ms.Length);
                            }
                        }
                    }
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.Message);

                    //throw; //todo
                }               
            }
        }
        private void xmiGridRestoreLayout_Click(object sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                #if USE_GROUP_PERSISTENCE
                    vRestoreLayoutFromIsolatedStorage(m_persistenceGroup);
                #else
                    vRestoreLayoutFromIsolatedStorage(m_persistenceSettings, xg);
                #endif
            }
        }
        #region Store Methods
        private static string sGetShowIsoStoreStatus()
        {

            // Obtain an isolated store for an application.
            try
            {
                using (var isfStore = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    string spaceUsed = (isfStore.Quota - isfStore.AvailableFreeSpace).ToString();
                    string spaceAvailable = isfStore.AvailableFreeSpace.ToString();
                    string curQuota = isfStore.Quota.ToString();
                    return
                        String.Format("Quota: {0} bytes, Used: {1} bytes, Available: {2} bytes",
                                curQuota, spaceUsed, spaceAvailable);
                }
            }

            catch (IsolatedStorageException)
            {
                return "Unable to access store.";

            }
        }
        private void vIncreaseStoreQuota(Int64 _iBytesToAdd)
        {
            // Obtain an isolated store for an application.
            try
            {
                using (var isfStore = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    // Request _iBytesToAdd more space in bytes.
                    Int64 curAvail = isfStore.AvailableFreeSpace;

                    // If available space is less than what is requested, try to increase.
                    //if (curAvail < _iBytesToAdd)
                    //{
                        // Request more quota space.
                        //if (!store.IncreaseQuotaTo(store.Quota + _iBytesToAdd))
                        //{
                        //    // The user clicked NO to the
                        //    // host's prompt to approve the quota increase.
                        //}
                        //else
                        //{
                        //    // The user clicked YES to the
                        //    // host's prompt to approve the quota increase.
                        //}
                        
                    //}
                    isfStore.IncreaseQuotaTo(isfStore.Quota + _iBytesToAdd);
                }
            }

            catch (IsolatedStorageException e)
            {
                // TODO: Handle that store could not be accessed.
                MessageBox.Show(e.Message);

            }
        }
#if USE_GROUP_PERSISTENCE
        private void vRestoreLayoutFromIsolatedStorage(PersistenceGroup _oSettings ) {
#else
        private void vRestoreLayoutFromIsolatedStorage(PersistenceSettings _oSettings, object _oControl) {
#endif         
            try
            {
                using (IsolatedStorageFile isfStore = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    if (isfStore.FileExists(sIsfName))
                    {
                        using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(sIsfName, FileMode.OpenOrCreate, isfStore))
                        {
                            try
                            {

                            #if USE_GROUP_PERSISTENCE
                                PersistenceManager.Load(_oSettings, stream);
                            #else
                                PersistenceManager.Load(_oControl as XamGrid, stream, _oSettings);
                            #endif
                            }
                            catch (Exception)
                            {
                                //throw; //todo
                            }

                        }
                    }
                }
            }
            catch (IsolatedStorageException e)
            {
                MessageBox.Show(e.Message);

                //throw;
            }
        }

        private void SaveInitialTestListSettings()
        {
            m_ms = PersistenceManager.Save(m_persistenceGroup);
        }
        #endregion Store Methods

        
        #endregion Persistence
        #endif

        #region Test Change

        #endregion // Test Change
        #region Data Retrieval Options
        private void xmiTestListDataRetrievalOptions_Click(object _sender, EventArgs _evArgs)
        {
            xdwTestListDataRetrievalOptions.WindowState = Infragistics.Controls.Interactions.WindowState.Normal;
        }
        private void evhbtnTestListDataRetrievalOptions_OK_Click(object _sender, RoutedEventArgs _evArgs)
        {
            m_vm.bTestListRetrieveRejectInfo = (rbIncludeRejectData.IsChecked.Value == true);

            xdwTestListDataRetrievalOptions.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
        }
        private void evhbtnTestListDataRetrievalOptions_Cancel_Click(object _sender, RoutedEventArgs _evArgs)
        {
            xdwTestListDataRetrievalOptions.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
        }     
        #endregion Data Retrieval Options
        #region BatchReports
        #region Full Rep        
        private void xmiBatchFullRep_Click(object _sender, EventArgs _evArgs)
        {
#if XDM_NAVIGATE_BY_INDEX
            Infragistics.Controls.Layouts.TabGroupPane tgpPane = mytgpList;
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#else
            Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#endif
            m_vm.sBusyText = ViewStrings.GeneratingReport;
            m_vm.bIsBusy = true;

            if ( xg.SelectionSettings.SelectedRows.Count > 0 ) {
                iBatchCount = xg.SelectionSettings.SelectedRows.Count;
                iBatchNdx = 0;
                xmiBatchFullRepEx();                
            }
            else {
                var mbResult = MessageBox.Show("Please select one or more tests", "Confirmation", MessageBoxButton.OK);
                m_vm.bIsBusy = false;
            }           
        }
        private void xmiBatchFullRepEx() 
        {
            if (iBatchNdx < iBatchCount)
            {
                m_vm.sBusyText = ViewStrings.GeneratingReport + " " + iBatchNdx.ToString() + "/" + iBatchCount.ToString();
                // Launch the next one
#if XDM_NAVIGATE_BY_INDEX
                Infragistics.Controls.Layouts.TabGroupPane tgpPane = mytgpList;
                Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#else
                Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
                Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#endif

#if SUPPORT_UNDO_REDO
                    ProductDataUR pd = null;
                #else
                    ProductData pd = null;
                #endif

                Row Item = xg.SelectionSettings.SelectedRows[iBatchNdx];
                
                #if SUPPORT_UNDO_REDO
                    pd = (ProductDataUR)Item.Data;
                #else
                    pd = (ProductData)Item.Data;
                #endif
                m_vm.vGetTestResults(new TestDescriptor(pd.sFamilyId, pd.dtTime, pd.dtTime.ToString(), pd.sStand, pd.sModel, pd.sSerial), (iBatchNdx <= (iBatchCount-1))?true:false);
                iBatchNdx++;
            }

        }
        #endregion Full Rep
        #endregion BatchReports
        #region Utility
        private void vSetupBindings(object _sender, NotificationEventArgs _evArgs)
        {
            // setup any bindings that I can't seem to make work through xaml.
            try
            {
                // update active pdb.
                Binding binding = new Binding("NAME"); //todo: this is not working properly???
                binding.Source = m_vm.SelectedPDBDefn;
                txtActivePDBName.SetBinding(TextBox.TextProperty, binding);
            }
            catch (Exception)
            {
                //throw; // todo:
            }

        }
        private void vResetTestListTweaks(object _sender, NotificationEventArgs _evArgs)
        {
#if XDM_NAVIGATE_BY_INDEX
            Infragistics.Controls.Layouts.TabGroupPane tgpPane = mytgpList;
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#else
                Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
                Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#endif

            xg.FilteringSettings.FilteringScope = Infragistics.Controls.Grids.FilteringScope.ColumnLayout;
            xg.FilteringSettings.RowFiltersCollection.Clear();
            xg.SortingSettings.SortedColumns.Clear();
            //xgTestList.SummaryRowSettings

        }
        private void vBuildVisibleTestList(Infragistics.Controls.Grids.XamGrid _xg)
        {
            //todo: this is really stupid to rebuild the entire list each time. can't I make a call from homevm to build it when I need it????
            if (m_vm.ocVisibleTests != null) m_vm.ocVisibleTests.Clear();
#if SUPPORT_UNDO_REDO
            m_vm.ocVisibleTests = new ObservableCollection<ProductDataUR>();
            ProductDataUR pd = null;
#else
            m_vm.ocVisibleTests = new ObservableCollection<ProductData>();
            ProductData pd = null;
#endif



            foreach (Infragistics.Controls.Grids.Row r in _xg.Rows)
            {
#if SUPPORT_UNDO_REDO
                pd = (ProductDataUR)(((Infragistics.Controls.Grids.RowBase)(r)).Data);
#else
                pd = (ProductData)(((Infragistics.Controls.Grids.RowBase)(r)).Data);
#endif
#if SUPPORT_UNDO_REDO                
                #if SUPPORT_TEST_LIST_BANDS
                    m_vm.ocVisibleTests.Add(new ProductDataUR(pd.sFamilyId, pd.dtTime, pd.sStand, pd.sModel, pd.sSerial, pd.sStatus,
                                                                        pd.colStatusList, pd.colRejList));
                #else
                    m_vm.ocVisibleTests.Add(new ProductDataUR(pd.sFamilyId, pd.dtTime, pd.sStand, pd.sModel, pd.sSerial, pd.sStatus));                
                #endif
#else
                #if SUPPORT_TEST_LIST_BANDS
                    m_vm.ocVisibleTests.Add(new ProductData(pd.sFamilyId, pd.dtTime, pd.sStand, pd.sModel, pd.sSerial, pd.sStatus, 
                                                            pd.colStatusList, pd.colRejList));
                #else
                    m_vm.ocVisibleTests.Add(new ProductData(pd.sFamilyId, pd.dtTime, pd.sStand, pd.sModel, pd.sSerial, pd.sStatus));                
                #endif

#endif
            }
            // tell view model I am done
            m_vm.vVisibleTestListBuilt();
        }
        private void vBuildVisibleTestList(object _sender, NotificationEventArgs _evArgs)
        {
#if XDM_NAVIGATE_BY_INDEX
            Infragistics.Controls.Layouts.TabGroupPane tgpPane = mytgpList;
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#else
                Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
                Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#endif

            vBuildVisibleTestList(xg);
        }
        #endregion // Utility
        #region Grouping
        private void evhxgTestList_GroupByChanged(object _sender, Infragistics.Controls.Grids.GroupByCollectionChangedEventArgs _evArgs)
        {
          
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)_sender;
            if (xg.GroupBySettings.GroupByOperation == GroupByOperation.GroupByRows) {

                #if SUPPORT_TEST_LIST_BANDS                    
                    foreach (object o in xg.Columns)
                    {
                        if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                        {
                            Column c = o as Column;
                            // if column was collapsed, ignore it
                            if (c.Visibility == System.Windows.Visibility.Visible)
                            {                                
                                if (_evArgs.NewGroupedColumns.Contains(c))
                                {
                                    c.Visibility = System.Windows.Visibility.Collapsed;
                                }
                                else
                                {
                                    c.Visibility = System.Windows.Visibility.Visible;
                                }
                            }
                        }
                    }
                #else
                    foreach (Infragistics.Controls.Grids.Column c in xg.Columns)
                    {
                        if (_evArgs.NewGroupedColumns.Contains(c))
                        {
                            c.Visibility = System.Windows.Visibility.Collapsed;
                        }
                        else
                        {
                            c.Visibility = System.Windows.Visibility.Visible;
                        }
                    }
                #endif
            }
        }
        private void xmiTestListShowGrouped_Click(object _sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sFamilyId"]);
                xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sModel"]);
                xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sStand"]);
                xg.GroupBySettings.GroupByOperation = GroupByOperation.GroupByRows;

                // close the context menu. Why doesn't this close automatically????
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
        private void xmiTestListShowMerged_Click(object _sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                Dispatcher.BeginInvoke(() =>
                {
                    xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sFamilyId"]);
                    xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sModel"]);
                    xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sStand"]);

#if SUPPORT_TEST_LIST_BANDS
                    foreach (object o in xg.Columns)
                    {
                        if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                        {
                            Column c = o as Column;
                            // if column was collapsed, ignore it
                            if (c.Visibility == System.Windows.Visibility.Visible)
                            {
                                c.Visibility = System.Windows.Visibility.Visible;
                            }
                        }
                    }
#else
                    foreach (Infragistics.Controls.Grids.Column c in xg.Columns)
                    {
                        c.Visibility = System.Windows.Visibility.Visible;
                    }
#endif

                    xg.GroupBySettings.GroupByOperation = GroupByOperation.MergeCells;
                });

                // close the context menu. Why doesn't this close automatically????
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
        #endregion // Grouping
        #region Saving

        private void OnGeneralAcknowledgmentNotice(object _sender, NotificationEventArgs<object, bool> _evArgs)
        {
            // Force acknowledgment from user.
            var mbResult = MessageBox.Show(_evArgs.Message, "Confirmation", MessageBoxButton.OK);

            // Call back ViewModel with response
            _evArgs.Completed(mbResult == MessageBoxResult.OK);
        }
        private void OnOverwritePDBNotice(object _sender, NotificationEventArgs<object, bool> _evArgs)
        {
            // Ask user if they want to overwrite a defined pdb?
            var mbResult = MessageBox.Show("Overwrite existing PDB?", "Confirmation", MessageBoxButton.OKCancel);

            // Call back ViewModel with response
            _evArgs.Completed(mbResult == MessageBoxResult.OK);
        }
        private void xmiSaveSelectedTests_Click(object _sender, EventArgs _evArgs)
        {
            //(Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList") 
#if XDM_NAVIGATE_BY_INDEX
            Infragistics.Controls.Layouts.TabGroupPane tgpPane = mytgpList;
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#else
                Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
                Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#endif

            if (xg.SelectionSettings.SelectedRows.Count > 0)
            {
                //????xgTestList.InvalidateData();
                m_vm.eSaveType = EPDBSaveTypes.eSELECTED;
                cbxOpenPDBOnSave.Visibility = System.Windows.Visibility.Visible;                
                // prompt for pdb name.
                xdwSavePDB.WindowState = Infragistics.Controls.Interactions.WindowState.Normal;
            }
            else
            {
                var mbResult = MessageBox.Show("Please select one or more tests", "Confirmation", MessageBoxButton.OK);
            }

        }
        private void xmiSaveAllTests_Click(object _sender, EventArgs _evArgs)
        {
#if XDM_NAVIGATE_BY_INDEX
            Infragistics.Controls.Layouts.TabGroupPane tgpPane = mytgpList;
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#else
                Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
                Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");
#endif

            if (xg.Rows.Count > 0)
            {
                //????xgTestList.InvalidateData();
                m_vm.eSaveType = EPDBSaveTypes.eALL;
                cbxOpenPDBOnSave.Visibility = System.Windows.Visibility.Collapsed;
                // prompt for pdb name.
                xdwSavePDB.WindowState = Infragistics.Controls.Interactions.WindowState.Normal;
            }
            else
            {
                var mbResult = MessageBox.Show("No tests.", "Confirmation", MessageBoxButton.OK);
            }
        }
        private void evhbtnSavePDB_OK_Click(object _sender, RoutedEventArgs _evArgs)
        {
            // Verify that user entered/selected a pdb.

            //  Regardless of whether use selects an existing pdb or enters a new one, the SelectedPDBDefn vm property is set
            if (xceSavePDBName.SelectedIndex > -1)
            {
                ProcessSavePDBOkClick();
                m_vm.PrevSelectedPDBDefn = m_vm.SelectedPDBDefn;
                if (m_vm.eSaveType != EPDBSaveTypes.eALL)
                {
                    // Does user want to open this saved pdb?  
                    m_vm.bOpenPDBOnSave = cbxOpenPDBOnSave.IsChecked == true;
                }
                else
                {
                    m_vm.bOpenPDBOnSave = false;
                }
            }
            else
            {
                var mbResult = MessageBox.Show("Please select a personal database", "Confirmation", MessageBoxButton.OK);
            }
        }
        private void evhbtnSavePDB_Cancel_Click(object _sender, RoutedEventArgs _evArgs)
        {
            xdwSavePDB.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
            // if user added one or more new "partial" pdb_defns, get rid of them.
            if (m_bNewPDB)
            {
                m_vm.vRemovePartialPDBDefns(true); //rrp
            }
            m_vm.SelectedPDBDefn = m_vm.PrevSelectedPDBDefn;
        }
        private void evhxdwSavePDB_WindowStateChanging(object _sender, Infragistics.Controls.Interactions.WindowStateChangingEventArgs _evArgs)
        {
            switch (_evArgs.NewWindowState)
            {
                case Infragistics.Controls.Interactions.WindowState.Normal:
                    m_bNewPDB = false;
                    //Infragistics.Controls.Interactions.XamDialogWindow xdwMe = (Infragistics.Controls.Interactions.XamDialogWindow)_sender;
                    m_vm.vSortPDBDefns();

                    // Set the selected item to the active pdb.
                    xceSavePDBName.SelectedItem = m_vm.SelectedPDBDefn;
                    break;
            }
        }
        private void xceSavePDBName_DataObjectRequested(object _sender, HandleableObjectGenerationEventArgs _evArgs)
        {

            //This event is raised, when the Infragistics.Controls.Editors.XamComboEditor needs to create a new data object. 
            SpecializedTextBox tbox = m_WMAMethods.FindChild(xceSavePDBName, typeof(SpecializedTextBox)) as SpecializedTextBox;
            // build a new "partial" pdb_defn
            _evArgs.NewObject = m_vm.oMakePartialPDBDefn(tbox.Text);
            _evArgs.Handled = true; // If e.Handled is set to true the ComboEditor will use the provided e.NewObject.

        }
        private void xceSavePDBName_ItemAdding(object _sender, ComboItemAddingEventArgs<ComboEditorItem> _evArgs)
        {
            //Occurs when an item is going to be added to the underlying ComboEditorItemCollection of the Infragistics.Controls.Editors.XamComboEditor  

            PDB_DEFN pdb = (PDB_DEFN)_evArgs.Item.Data; //IG issue 90008 pdb.USER_NAME gets "nulled" if _evArgs.Handled is not set to true and even
            // if it is, according to them, that could change in the future so slam it back in anyway
            pdb.USER_NAME = m_vm.sUserName;
        }
        private void xceSavePDBName_ItemAdded(object _sender, ComboItemAddedEventArgs<ComboEditorItem> e)
        {
            //Occurs when an item is added to the underlying ComboEditorItemCollection of the Infragistics.Controls.Editors.XamComboEditor
            m_bNewPDB = true;
        }
        private void ProcessSavePDBOkClick()
        {
            xdwSavePDB.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;

            // User could have entered multiple new pdbdef names prior to striking the ok button, so make sure
            // to remove all of these "partial" pdbDefns. NOTE: Remove all but the "selected" one which is the one they want.

            // if user added one or more new "partial" pdb_defns, get rid of them.  
            m_vm.vRemovePartialPDBDefns(false);

            switch (m_vm.eSaveType)
            {
                case EPDBSaveTypes.eALL:
                    m_vm.vSaveAllTests();
                    break;
                case EPDBSaveTypes.eSELECTED:
                    m_vm.vSaveSelectedTestsToPDB();
                    break;
                default:
                    break;
            }
        }
        private void xceSavePDBName_SelectionChanged(object _sender, EventArgs _evArgs)
        {
            var xce = _sender as XamComboEditor;

            if (xce != null)
            {
                m_vm.SelectedPDBDefn = xce.SelectedItem as PDB_DEFN;
            }
            else
            {
                //todo
            }
        }

        #endregion //Saving
        #region Opening

        private void xmiOpenPDB_Click(object _sender, EventArgs _evArgs)
        {
            xdwOpenPDB.WindowState = Infragistics.Controls.Interactions.WindowState.Normal;
        }
        private void evhxdwOpenPDB_WindowStateChanging(object _sender, Infragistics.Controls.Interactions.WindowStateChangingEventArgs _evArgs)
        {
            switch (_evArgs.NewWindowState)
            {
                case Infragistics.Controls.Interactions.WindowState.Normal:
                    m_bNewPDB = false;
                    // stuff the working pdb name into the pdb name box.
                    Infragistics.Controls.Interactions.XamDialogWindow xdwMe = (Infragistics.Controls.Interactions.XamDialogWindow)_sender;

                    m_vm.vSortPDBDefns();
                    // set the selected item to the "Active" pdb.
                    xceOpenPDBName.SelectedItem = m_vm.SelectedPDBDefn;
                    break;
            }
        }
        private void evhbtnOpenPDB_OK_Click(object _sender, RoutedEventArgs _evArgs) //rrp
        {
            // Verify that user chose a pdb.
            if (xceOpenPDBName.SelectedItems.Count > 0)
            {
                xdwOpenPDB.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
                // open the selected pdbdefn
                m_vm.vOpenPDB();
                m_vm.PrevSelectedPDBDefn = m_vm.SelectedPDBDefn;
            }
            else
            {
                var mbResult = MessageBox.Show("Please select a personal database", "Confirmation", MessageBoxButton.OK);
            }
        }
        private void evhbtnOpenPDB_Cancel_Click(object _sender, RoutedEventArgs _evArgs)
        {
            xdwOpenPDB.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
            m_vm.SelectedPDBDefn = m_vm.PrevSelectedPDBDefn;
        }
        private void xceOpenPDBName_SelectionChanged(object _sender, EventArgs _evArgs)
        {
            var xce = _sender as XamComboEditor;

            if (xce != null)
            {
                m_vm.SelectedPDBDefn = xce.SelectedItem as PDB_DEFN;
            }
            else
            {
                //todo
            }
        }
        private void evhbtnPdbGetData_Click(object _sender, RoutedEventArgs _evArgs)
        {
            
            // Verify that user chose a pdb.
            if (cmbPdbName.SelectedIndex >= 0)
            {             
                // open the selected pdbdefn
                m_vm.vOpenPDB();
                m_vm.PrevSelectedPDBDefn = m_vm.SelectedPDBDefn;
            }
            else
            {
                var mbResult = MessageBox.Show("Please select a personal database", "Confirmation", MessageBoxButton.OK);
            }
        }
        #endregion //Opening
        #region Filtration
        private void btnGetData_Click(object _sender, RoutedEventArgs _evArgs)
        {
            // if no families are selected then tell the user to pick at least 1.

            if (cmbFamily.SelectedItems.Count > 0)
            {
                m_vm.evhGetProductData(cmbStand, cmbFamily, cmbModels);
            }
            else
            {
                MessageBox.Show("Selected at least ONE family before trying to get data.");
            }

        }
        #endregion // Filtration
        #region Export

        private void evhbtnExportOptions_OK_Click(object _sender, RoutedEventArgs _evArgs)
        {
            //todo: all the the export properties can be placed on a dialog window with checkboxes so user can choose.

            //x xgExcelExporter.ExportRange = ExcelExportRange.All;
            //ExcludeColumnFormatString 	bool 	Controls whether a column’s format string is applied to the exported data. 	false
            //ExcludeColumnValueConverter 	bool 	Controls whether a column’s value converter is applied to the exported data. 	false
            //ExcludeExpandedLayouts 	bool 	Controls whether the available child bands are exported. 	false
            //ExcludeFilters 	bool 	Controls whether the current filters are applied to the exported data. 	false
            //ExcludeFixedColumns 	bool 	Controls whether the fixed columns are exported. 	false
            //ExcludeGroupBy 	bool 	Controls whether grouping are applied to the exported data. 	false
            //ExcludeHiddenColumns 	bool 	Controls whether the xamGrid hidden columns are exported. 	false
            //ExcludeSort 	bool 	Controls whether the current sort order of the xamGrid data are exported. 	false
            //ExcludeSummaries 	bool 	Controls whether the summaries are exported. 	false
            //ExcludeTemplateColumns 	bool 	Controls whether the template columns are exported. 	false
            //ExcludeUnboundColumns 	bool 	Controls whether the unbound columns are exported. 	false             

            //xgExcelExporter.ExportLayoutSettings.
            //ColumnLayoutHeaderRowCellFormat 	WorksheetCellFormatSettings 	Use this property to set formatting (alignment, fill pattern, borders and font setting) for every column layout header row’s cell.
            //DataRowCellFormat 	WorksheetCellFormatSettings 	Use this property to set formatting for every data row’s cell.
            //FooterRowCellFormat 	WorksheetCellFormatSettings 	Use this property to set formatting for every footer row’s cell.
            //GroupByRowCellFormat 	WorksheetCellFormatSettings 	Use this property to set formatting for every group by row’s cell.
            //HeaderRowCellFormat 	WorksheetCellFormatSettings 	Use this property to set formatting for every header row’s cell.

            //xgExcelExporter.ExportEnded += new System.EventHandler<System.EventArgs>(evhTestList_ExcelExporter_ExportEnded);
            if (cbxRangeAllOrCurrentPage.IsChecked == true)
            {
                ExcelExporter.ExportRange = ExcelExportRange.All;
            }
            else
            {
                ExcelExporter.ExportRange = ExcelExportRange.CurrentPage;
            }
            ExcelExporter.ExcludeColumnFormatString = (cbxExcludeColumnFormatString.IsChecked == true);
            ExcelExporter.ExcludeColumnValueConverter = (cbxExcludeColumnValueConverter.IsChecked == true);
            ExcelExporter.ExcludeExpandedLayouts = (cbxExcludeExpandedLayouts.IsChecked == true);
            ExcelExporter.ExcludeFilters = (cbxExcludeFilters.IsChecked == true);
            ExcelExporter.ExcludeFixedColumns = (cbxExcludeFixedColumns.IsChecked == true);
            ExcelExporter.ExcludeGroupBy = (cbxExcludeGroupBy.IsChecked == true);
            ExcelExporter.ExcludeHiddenColumns = (cbxExcludeHiddenColumns.IsChecked == true);
            ExcelExporter.ExcludeSort = (cbxExcludeSort.IsChecked == true);
            ExcelExporter.ExcludeSummaries = (cbxExcludeSummaries.IsChecked == true);
            ExcelExporter.ExcludeTemplateColumns = (cbxExcludeTemplateColumns.IsChecked == true);
            ExcelExporter.ExcludeUnboundColumns = (cbxExcludeUnboundColumns.IsChecked == true);

            xdwExportGridOptions.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;

        }
        private void evhbtnExportOptions_Cancel_Click(object _sender, RoutedEventArgs _evArgs)
        {
            xdwExportGridOptions.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
        }
        private void xmiExportGridOptions_Click(object _sender, EventArgs _evArgs)
        {
            xdwExportGridOptions.WindowState = Infragistics.Controls.Interactions.WindowState.Normal;
        }
        private void xmiExportGrid_Click(object _sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;

            if (xg != null)
            {
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;

                if (xg.Rows.Count <= 0)
                {
                    var mbResult = MessageBox.Show("No information to export", "Confirmation", MessageBoxButton.OK);
                }
                else
                {
                    SaveFileDialog dlgSaveFile = new SaveFileDialog();
                    dlgSaveFile.DefaultExt = "xlsx";
                    dlgSaveFile.Filter = "Excel Workbook (.xlsx)|*.xlsx";

                    if (dlgSaveFile.ShowDialog().Value)
                    {
                        try
                        {
                            m_stream = dlgSaveFile.OpenFile();

                            // Create a Workbook object with one Worksheet
                            m_Workbook = new Workbook();
                            Worksheet worksheet = m_Workbook.Worksheets.Add("Sheet1");
                            m_Workbook.SetCurrentFormat(WorkbookFormat.Excel2007);
#if SUPPORT_UNDO_REDO
                        m_vm.oProductDataURVM.UndoManager.Suspend();
#endif

                            // Export the xamGrid data to the Workbook object
                            this.ExcelExporter.ExportAsync(xg, m_Workbook);
                        }
                        catch (Exception exc)
                        {
                            MessageBox.Show(exc.Message);
                        }
                    }
                }
            }
        }
        private void evhTestList_ExcelExporter_ExportEnded(object _sender, EventArgs _evArgs)
        {
            try
            {
                // Save the file using the Save method of the Workbook object.
                m_Workbook.Save(m_stream);
            }
            catch (Exception)
            {

                //throw; //todo
            }
            finally
            {
                m_stream.Close();
                m_vm.sBusyText = ApplicationStrings.BusyIndicatorPleaseWait;
                m_vm.bIsBusy = false;
#if SUPPORT_UNDO_REDO
                m_vm.oProductDataURVM.UndoManager.Resume();
#endif

            }
        }
        private void evhTestList_ExcelExporter_ExportStarted(object _sender, ExcelExportStartedEventArgs _evArgs)
        {
            m_vm.sBusyText = ApplicationStrings.BusyIndicatorPleaseWait;
            m_vm.bIsBusy = true;
        }
        private void evhTestList_ExcelExporter_RowExporting(object _sender, XamGridExcelExporterRowExportingEventArgs _evArgs)
        {
            m_vm.sBusyText = ViewStrings.ExportingRow + " " + (_evArgs.Row.Index + 1).ToString();
        }
        private void evhTestList_ExcelExporter_CellExporting(object _sender, XamGridExcelExporterCellExportingEventArgs _evArgs)
        {
            //XamGridExcelExporter ee = (XamGridExcelExporter)sender;
            //Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenusXamGrid) as XamGrid;
            switch (sContextMenuXamGrid)
            {
                case "xgTestList":
                    // xgTestList has an image column that needs special care.
                    switch (_evArgs.Column.Key)
                    {
                        case "sStatus":
#if SHOW_IMAGE
                    // Handle image export
                    // currently exported cell 
                    
                    WorksheetCell cell = _evArgs.CurrentWorksheet.Rows[_evArgs.CurrentRowIndex].Cells[_evArgs.CurrentColumnIndex];
                    WorksheetCell diagonalCell = _evArgs.CurrentWorksheet.Rows[_evArgs.CurrentRowIndex + 1].Cells[_evArgs.CurrentColumnIndex + 1];

                    // current worksheet 
                    Worksheet sheet = _evArgs.CurrentWorksheet;

                    // 1. Create Image 
                    // resolve your path image 
                    string imagePath = ResolveImagePathFromCellValue(_evArgs.Value);
                    System.Drawing.Image image = new System.Drawing.Bitmap(imagePath);
                    WorksheetImage workImage = new WorksheetImage((image));

                    // 2. Position the Image 

                    workImage.TopLeftCornerCell = cell;
                    workImage.BottomRightCornerCell = diagonalCell;

                    // 3. Insert Image 
                    sheet.Shapes.Add(workImage);
                    cell.Value = null;
#else
#if SUPPORT_UNDO_REDO
                        string sTestStatus = (_evArgs.Row.Data as ProductDataUR).sStatus;
#else
                            string sTestStatus = (_evArgs.Row.Data as ProductData).sStatus;
#endif

                            //todo: remove hardcoded "4" and "5"
                            if (sTestStatus == "4")
                            {
                                _evArgs.ExportObject = "Accept";
                            }
                            else
                            {
                                _evArgs.ExportObject = "Reject";
                            }
#endif
                            break;
                    }
                    break;

                default:
                    break;
            }
        }
        
        #endregion
        #region Expand Collapse
        private void xmiTestListExpandAll_Click(object _sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                foreach (Row r in xg.Rows)
                {
                    r.IsExpanded = true;

                }
            }
        }
        private void xmiTestListCollapseAll_Click(object _sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                foreach (Row r in xg.Rows)
                {
                    r.IsExpanded = false;

                }
            }
        }

        #endregion Expand Collapse

        #region Context Menu
        private void cmnuTestList_Item_Clicked(object _sender, ItemClickedEventArgs _evArgs)
        {
            // menu level click handler
            // IG:95417 Make context menu invisibible when item is choosen until IG bug is fixed. 
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }   
        #endregion Context Menu
        #endregion Tests

        #region TestPoints
        #region Events
#if SUPPORT_TEST_POINT_LIST_RESIZE
        private void evhxgTPList_InitializeRow(object _sender, InitializeRowEventArgs _evArgs)
        {
            // This check keeps the pane from performing these operations when a new row is added due
            // to virtualization.
            if (!_bTPListPaneReady)
            {

                // It's ok to do this every frame since each subsequent row can only increase the column width.
                double totalWidth = 0;

                // Calculate the total width taken up by all the columns.
                foreach (Column col in (_sender as XamGrid).Columns)
                    totalWidth += col.ActualWidth;

                // Now calculate the percentages for each of the columns.
                foreach (Column col in (_sender as XamGrid).Columns)
                {
                    ColumnInfo cInfo = new ColumnInfo();
                    cInfo.dRelativeWidth = col.ActualWidth / totalWidth;
                    cInfo.dWidth = col.ActualWidth;

                    // Store it for later use.
                    col.Tag = cInfo;
                }

                // Use a dispatcher to delay setting the panes width.
                Dispatcher.BeginInvoke(() =>
                {
#if XDM_NAVIGATE_BY_INDEX
//                    SplitPane myspPDBx = dmPDB.Panes[0] as SplitPane;
//                    TabGroupPane mytgpListx = myspPDB.Panes[0] as TabGroupPane;
                    TabGroupPane mytgpListx = dmPDB.Panes[0] as TabGroupPane;
                    ContentPane mycpTestListx = mytgpList.Panes[0] as ContentPane;
                    ContentPane pane = mycpTestListx;
#else
                    ContentPane pane = dmPDB.FindPane("cpTPList") as ContentPane;
#endif
                    if (pane != null)
                    {
                        // Add some extra padding.
                        pane.FloatWidth = totalWidth + dScrollBarWidth;
                    }
                    // The order in which things occur is that this dispatcher code will be executed a number of times for each row initialized.
                    // This will also set calculating to false.  Once these initial rows are finished, the SizeChanged event on the pane will
                    // be executed.
                    _bTPListStillCalculating = false;
                });
            }
        }
        private void evhxgTPList_CellDoubleClicked(object _sender, CellClickedEventArgs _evArgs)
        {
            if (_evArgs.Cell.Row.Data.GetType() != typeof(TpDefn)) return;
            iBatchNdx = iBatchCount = 0; // we are not doing batch style           
            TpDefn pd = _evArgs.Cell.Row.Data as TpDefn;
            m_vm.vGetTPData(new TPDescriptor(pd.dTpDefnSakey, pd.dId, pd.sText, pd.sUnits));
        }      
#endif
        private void evhxgTPList_MouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {   // IG:95417 Make context menu visibible when right click on grid occurs until IG bug is fixed. 
            Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(sender as XamGrid);
            cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Visible;
        }
        #endregion Events
        #region TestPoint Change
        private void cmnuTPList_Item_Clicked(object _sender, ItemClickedEventArgs _evArgs)
        {
            // menu level click handler
            // IG:95417 Make context menu invisibible when item is choosen until IG bug is fixed. 
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
        #endregion //TestPoint Change
        #region Retrieve
        private void xmiRetrieveTPDefns_Click(object _sender, EventArgs _evArgs)
        {
      

            m_vm.vRetrieveTPDefns();
            // make the test point list active. 
#if XDM_NAVIGATE_BY_INDEX          
            TabGroupPane tgpList = mytgpList;
#else
                        Infragistics.Controls.Layouts.TabGroupPane tgpList = (Infragistics.Controls.Layouts.TabGroupPane)dmPDB.FindPane("tgpList");
#endif
            if (tgpList != null)
            {
                foreach (ContentPane cp in tgpList.Panes)
                {
                    if (cp.Name == "cpTPList")
                    {
                        // Make pane the active one and exit.
                        cp.IsActivePane = true;
                        break;
                    }
                }
            }
        }
        #endregion // Retrieve
        #region Grouping
        private void evhxgTPList_GroupByChanged(object _sender, Infragistics.Controls.Grids.GroupByCollectionChangedEventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)_sender;
            #if SUPPORT_TEST_LIST_BANDS                    
                foreach (object o in xg.Columns)
                {
                    if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                    {
                        Column c = o as Column;
                        // if column was collapsed, ignore it
                        if (c.Visibility == System.Windows.Visibility.Visible)
                        {
                            if (_evArgs.NewGroupedColumns.Contains(c))
                            {
                                c.Visibility = System.Windows.Visibility.Collapsed;
                            }
                            else
                            {
                                c.Visibility = System.Windows.Visibility.Visible;
                            }
                        }
                    }
                }
            #else
                foreach (Infragistics.Controls.Grids.Column c in xg.Columns)
                {
                    if (_evArgs.NewGroupedColumns.Contains(c))
                    {
                        c.Visibility = System.Windows.Visibility.Collapsed;
                    }
                    else
                    {
                        c.Visibility = System.Windows.Visibility.Visible;
                    }
                }
            #endif
        }
        #endregion //Grouping
        #region Options        
       
        #endregion // Options
        #region Batch Reports
        #region TP Data

        private void xmiHistogram_Click(object _sender, EventArgs _evArgs)
        {
            vBatchTPData(_sender, _evArgs, 2);
        }

        private void xmiTrend_Click(object _sender, EventArgs _evArgs)
        {
            vBatchTPData(_sender, _evArgs, 1);
        }

        private void vBatchTPData(object _sender, EventArgs _evArgs, int iChartType )
        {
            m_vm.sBusyText = ViewStrings.GeneratingChart;
            m_vm.bIsBusy = true;

#if XDM_NAVIGATE_BY_INDEX
            TabGroupPane tgpPane = mytgpList;
            XamGrid xg = (XamGrid)tgpPane.FindName("xgTPList");
#else
                Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
                Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTPList");
#endif
            if (xg.SelectionSettings.SelectedRows.Count > 0)
            {
                iBatchCount = xg.SelectionSettings.SelectedRows.Count;
                iBatchNdx = 0;
                xmiBatchTPDataEx( iChartType );
            }
            else
            {
                var mbResult = MessageBox.Show("Please select one or more test points", "Confirmation", MessageBoxButton.OK);
                m_vm.bIsBusy = false;
            }
        }

        private void xmiBatchTPDataEx( int iType )
        {
            if (iBatchNdx < iBatchCount)
            {
                m_vm.sBusyText = ViewStrings.GeneratingChart + " " + (iBatchNdx + 1).ToString() + "/" + iBatchCount.ToString();
                // Launch the next one
#if XDM_NAVIGATE_BY_INDEX
                TabGroupPane tgpPane = mytgpList;
                XamGrid xg = (XamGrid)tgpPane.FindName("xgTPList");
#else
                    Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
                    Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTPList");
#endif

                TpDefn tpd = null;

                Row Item = xg.SelectionSettings.SelectedRows[iBatchNdx];
                tpd = (TpDefn)Item.Data;
                m_vm.vGetTPData(new TPDescriptor(tpd.dTpDefnSakey, tpd.dId, tpd.sText, tpd.sUnits, iType), (iBatchNdx <= (iBatchCount - 1)) ? true : false);
                iBatchNdx++;
            }
           
        }
        #endregion TP Data

        #endregion Batch Reports
        #region Pivot Grid

        private void xmiShowTPPivotGrid_Click(object sender, EventArgs e) 
        {
#if SUPPORT_UNDO_REDO
#if XDM_NAVIGATE_BY_INDEX
            ContentPane cpMyTestList = mycpTestList;   
#else
            ContentPane cpMyTestList = dmPDB.FindPane("cpTestList") as ContentPane;   
#endif
            Grid grdParent = (Grid)cpMyTestList.Content;
            XamGrid xgMyTestList = (XamGrid)grdParent.Children[1];

            XamGrid xgMyTPList = dmPDB.FindName("xgTPList") as XamGrid;
#else
            ContentPane cpMyTestList = dmPDB.FindPane("cpTestList") as ContentPane;
            XamGrid xgMyTestList = cpMyTestList.Content as XamGrid;

            XamGrid xgMyTPList = dmPDB.FindName("xgTPList") as XamGrid;
#endif

            // if none selected, default to all if (xgMyTestList.SelectionSettings.SelectedRows.Count > 0 && xgMyTPList.SelectionSettings.SelectedRows.Count > 0)
            // if none selected, default to all {
#if GET_ACTUAL_PIVOT_DATA
                m_vm.vGetTPPivotGridData();
#else
                List<string> lstTests = new List<string>();
                List<decimal> lstTPIds = new List<decimal>();
#if SUPPORT_UNDO_REDO
                ProductDataUR pd = null;
#else
                ProductData pd = null;
#endif
                // build list of tests and test points that were choosen
                // make a list of testdts/standcode aggregations
                foreach (Infragistics.Controls.Grids.Row Item in xgMyTestList.SelectionSettings.SelectedRows)
                {
#if SUPPORT_UNDO_REDO
                    pd = (ProductDataUR)Item.Data;
#else
                    pd = (ProductData)Item.Data;
#endif

                    lstTests.Add(pd.dtTime.ToString("yyyy MM dd hh mm ss") + " " + pd.sStand); // this will be reconstructed on domainservice side since you are 
                    // required to pass in to domain services ONLY SERIALIZABLE types.
                }
                TpDefn tp = null;
                foreach (Infragistics.Controls.Grids.Row Item in xgMyTPList.SelectionSettings.SelectedRows)
                {
                    tp = (TpDefn)Item.Data;
                    lstTPIds.Add(tp.dId); 
                }
                m_vm.vGetTPPivotGridData(lstTests, lstTPIds);

#endif

            // if none selected, default to all }
                // if none selected, default to all else
            // if none selected, default to all {
            // if none selected, default to all     var mbResult = MessageBox.Show("Please select one or more tests and test points", "Confirmation", MessageBoxButton.OK);
            // if none selected, default to all }

        }

        #region Context Menu
        private void cmnuPivotGrid_Item_Clicked(object _sender, ItemClickedEventArgs _evArgs)
        {
            // menu level click handler
            // IG:95417 Make context menu invisibible when item is choosen until IG bug is fixed. 
            Infragistics.Controls.Grids.XamPivotGrid xpg = dmPDB.FindName(sContextMenuXamGrid) as XamPivotGrid;
            if (xpg != null)
            {
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xpg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
        #endregion Context Menu

        #region Export

        private void evhbtnExportPivotGridOptions_OK_Click(object _sender, RoutedEventArgs _evArgs)
        {       
            xdwExportPivotGridOptions.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
        }
        private void evhbtnExportPivotGridOptions_Cancel_Click(object _sender, RoutedEventArgs _evArgs)
        {
            xdwExportPivotGridOptions.WindowState = Infragistics.Controls.Interactions.WindowState.Hidden;
        }
        private void xmiExportPivotGridOptions_Click(object _sender, EventArgs _evArgs)
        {
            xdwExportPivotGridOptions.WindowState = Infragistics.Controls.Interactions.WindowState.Normal;
        }
        private void xmiExportPivotGrid_Click(object _sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamPivotGrid xpg = dmPDB.FindName(sContextMenuXamGrid) as XamPivotGrid;

            if (xpg != null)
            {
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xpg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;

                if (xpg.GridLayout != null && xpg.GridLayout.Rows != null && xpg.GridLayout.Columns != null )
                {
               
                    SaveFileDialog dlgSaveFile = new SaveFileDialog();
                    dlgSaveFile.DefaultExt = "xlsx";
                    dlgSaveFile.Filter = "Excel Workbook (.xlsx)|*.xlsx";

                    if (dlgSaveFile.ShowDialog().Value)
                    {
                        try
                        {
                            m_stream = dlgSaveFile.OpenFile();

                            // Create a Workbook object with one Worksheet
                            //m_Workbook = new Workbook();
                            m_Workbook = oCreatePivotGridWorkbook(xpg);
                            //Worksheet worksheet = m_Workbook.Worksheets.Add("Sheet1");
                            //m_Workbook.SetCurrentFormat(WorkbookFormat.Excel2007);

                            // Export the xamGrid data to the Workbook object
                            m_vm.sBusyText = ApplicationStrings.BusyIndicatorPleaseWait;
                            m_vm.bIsBusy = true;
                            try
                            {
                                // Save the file using the Save method of the Workbook object.
                                m_Workbook.Save(m_stream);
                            }
                            catch (Exception)
                            {
                                //throw; //todo
                            }
                            finally
                            {
                                m_stream.Close();
                                m_vm.sBusyText = ApplicationStrings.BusyIndicatorPleaseWait;
                                m_vm.bIsBusy = false;
                            }
                        }
                        catch (Exception exc)
                        {
                            MessageBox.Show(exc.Message);
                        }
                    }
                }
                else {
                    var mbResult = MessageBox.Show("No information to export", "Confirmation", MessageBoxButton.OK);
                }
            }
        }
        private Workbook oCreatePivotGridWorkbook( XamPivotGrid _xpg)
        {
            Boolean CompactLayoutEnabled = _xpg.AllowCompactLayout;

            if (CompactLayoutEnabled)
                _xpg.AllowCompactLayout = false;

            //Calculate the dimensions of the header rows and columns
            int TopHeaderHeight = 0;
            if (cbxExportColumnHeaders.IsChecked.Value)
            {
                TopHeaderHeight = _xpg.GridLayout.PrivateRowsForColumnPanel.Count;
            }

            int LeftHeaderWidth = 0;
            if (cbxExportRowHeaders.IsChecked.Value) 
            {
                LeftHeaderWidth = _xpg.GridLayout.PrivateColumnsForRowPanel.Count;
            }

            //Create the blank workbook and the sheet in it that will contain the PivotGrid data
            Workbook dataWorkbook = new Workbook();
            Worksheet sheetOne = dataWorkbook.Worksheets.Add("Sheet1");
            dataWorkbook.SetCurrentFormat(WorkbookFormat.Excel2007);

            //Freeze header rows and columns
            if (cbxExportColumnHeaders.IsChecked.Value || cbxExportRowHeaders.IsChecked.Value)
                sheetOne.DisplayOptions.PanesAreFrozen = true;
            if (cbxExportRowHeaders.IsChecked.Value)
                sheetOne.DisplayOptions.FrozenPaneSettings.FrozenRows = TopHeaderHeight;
            if (cbxExportColumnHeaders.IsChecked.Value)
                sheetOne.DisplayOptions.FrozenPaneSettings.FrozenColumns = LeftHeaderWidth;

            //Set default width for the columns
            sheetOne.DefaultColumnWidth = 5000;


            int ColumnSpan, ColumnId, RowSpan, RowId;
            string CellValue;

            if (cbxExportColumnHeaders.IsChecked.Value)
            {
                // Build Column Header
                foreach (PivotColumnHeaderCell cell in _xpg.GridLayout.ColumnHeaderCells)
                {
                    ColumnSpan = cell.RealColumnSpan;
                    ColumnId = _xpg.GridLayout.Columns
                        .IndexOf(cell.Column as PivotDataColumn)
                        + LeftHeaderWidth;

                    RowSpan = cell.RowSpan;
                    RowId = _xpg.GridLayout
                        .PrivateRowsForColumnPanel.IndexOf(cell.Row as PivotHeaderRow);

                    CellValue = cell.Member.Caption;
                    if (cell.Member.IsTotal && cell.IsToggleVisible == false)
                        CellValue += " " + ViewStrings.Total;

                    if (ColumnSpan > 1 || RowSpan > 1)
                        sheetOne.MergedCellsRegions.Add(
                            RowId,
                            ColumnId,
                            RowId + RowSpan - 1,
                            ColumnId + ColumnSpan - 1
                            );

                    this.SetCellValue(sheetOne.Rows[RowId].Cells[ColumnId], CellValue, "TopHeader");
                }
            }

            if (cbxExportRowHeaders.IsChecked.Value)
            {
                // Build Row Header
                foreach (PivotRowHeaderCell cell in _xpg.GridLayout.RowHeaderCells)
                {
                    ColumnSpan = cell.ColumnSpan;
                    ColumnId = _xpg.GridLayout
                        .PrivateColumnsForRowPanel.IndexOf(cell.Column as PivotHeaderColumn);

                    RowSpan = cell.RealRowSpan;
                    RowId = _xpg.GridLayout
                        .Rows.IndexOf(cell.Row as PivotDataRow) + TopHeaderHeight;

                    CellValue = cell.Member.Caption;
                    if (cell.Member.IsTotal && cell.IsToggleVisible == false)
                        CellValue += " " + ViewStrings.Total;

                    if (ColumnSpan > 1 || RowSpan > 1)
                        sheetOne.MergedCellsRegions.Add(
                            RowId,
                            ColumnId,
                            RowId + RowSpan - 1,
                            ColumnId + ColumnSpan - 1
                            );

                    this.SetCellValue(sheetOne.Rows[RowId].Cells[ColumnId], CellValue, "LeftHeader");
                }
            }

            // Build Data
            int currentRow = 0;
            foreach (PivotDataRow row in _xpg.GridLayout.Rows)
            {
                int currentCell = 0;
                foreach (PivotCell cell in row.Cells)
                {
                    string currentCellValue;
                    if (cell.Data != null)
                    {
                        currentCellValue = (cell.Data as ICell).FormattedValue;
                    }
                    else
                    {
                        currentCellValue = string.Empty;
                    }
                    this.SetCellValue(
                        sheetOne.Rows[currentRow + TopHeaderHeight]
                            .Cells[currentCell + LeftHeaderWidth],
                        currentCellValue,
                        "Data");
                    currentCell++;
                }
                currentRow++;
            }


            if (CompactLayoutEnabled)
                _xpg.AllowCompactLayout = true;
            //return the filled in workbook
            return dataWorkbook;

        }

        //Set the value and format each cell
        private void SetCellValue(WorksheetCell cell, string cellValue, string cellType)
        {
            cell.Value = cellValue;
            cell.CellFormat.ShrinkToFit = ExcelDefaultableBoolean.True;
            cell.CellFormat.VerticalAlignment = VerticalCellAlignment.Center;
            cell.CellFormat.Alignment = HorizontalCellAlignment.Center;

            if (cellType == "TopHeader")
            {
                //cell.CellFormat.Fill = CellFill.CreatePatternFill(ColorMaker.GetColor(255, 181, 212, 240), ColorMaker.GetColor(255, 255, 255, 255), FillPatternStyle.Solid);
            }
            else if (cellType == "LeftHeader")
            {
                //cell.CellFormat.Fill = CellFill.CreatePatternFill(ColorMaker.GetColor(255, 200, 212, 240), ColorMaker.GetColor(255, 255, 255, 255), FillPatternStyle.Solid);
            }
            else if (cellType == "Data")
            {
                //cell.CellFormat.Fill = CellFill.CreatePatternFill(ColorMaker.GetColor(255, 242, 244, 249), ColorMaker.GetColor(255, 255, 255, 255), FillPatternStyle.Solid);
            }
            else
            {
                //MessageBox.Show(PivotGridStrings.XPG_Cell_Type_Not_Located);
            }
        }
        #endregion
        #endregion Pivot Grid

        #endregion TestPoints

        #region Data

        private bool boActivatePane(string sName)
        {
            ContentPaneEx cpex = null;

            // check closed panes first
            if (dmPDB.ClosedPanes.Count > 0)
            {
                foreach (ContentPaneEx cp in dmPDB.ClosedPanes)
                {
                    if (cp.Name == sName)
                    {
                        cpex = cp;
                        break;
                    }
                }
            }
            else
            {
                foreach (ContentPaneEx cp in mytgpData.Panes)
                {
                    if (cp.Name == sName)
                    {
                        cpex = cp;
                        break;
                    }
                }
            }

            // panes and closedPane collection. 
            if (cpex != null)
            {
                cpex.OpenPane(); //when you call OpenPane on a pane that is currently in the ClosedPanes collection, it will add the pane to the TabGroupPane it previously belonged to
                if (cpex.Content != null) // when a pane is closed, I destroy the content of the pane so as not to hog memory.
                {
                    // Make pane the active one and exit.
                    cpex.IsActivePane = true;
                    return true;
                }
            }

            return false;
        }

        private ContentPaneEx vCreateContentPane( string sName, string sHeader, string sTabHeader, string sImagePath, string sToolTipTemplate )
        {
            ContentPaneEx cpex = null;

            var ct = (ControlTemplate)this.Resources[sToolTipTemplate];

            cpex = new ContentPaneEx();
            // tab name
            cpex.Name = sName;
            cpex.AllowClose = true;
            cpex.TabHeaderToolTipTemplate = ct;

            // In order to get an image to show up in the tab, a new datatemplate has to be created. Since I am building the contentpane in code,
            // I also have to build the datatemplate in code as well.

            //Build a DataTemplate for the pane's header.               
            cpex.HeaderTemplate = BuildHeaderTemplate(cpex, "Header", "Image");
            // Build a DataTemplate for the pane's tab header.
            cpex.TabHeaderTemplate = BuildHeaderTemplate(cpex, "TabHeader", "Tag");

            // Tab header/text
            cpex.TabHeader = sTabHeader;
            // Pane header text
            cpex.Header = sHeader;
            // Tag header image
            cpex.Tag = sImagePath;
            // Pane header image
            cpex.Image = new BitmapImage(new Uri(sImagePath, UriKind.Relative));

            cpex.IsPinned = true;
            cpex.IsDocumentPane = true;
            cpex.IsActivePane = false;
            cpex.IsMaximized = false;
            cpex.CloseButtonVisibility = System.Windows.Visibility.Visible;
            cpex.PinButtonVisibility = System.Windows.Visibility.Collapsed;

            return cpex;
        }

        #region Full Rep
   
        // Programatically construct a datatemplate. Used by xgFullRep
        private DataTemplate BuildHeaderTemplate(ContentPane _cpPane, string _sTextPath, string _sImagePath )
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("<DataTemplate xmlns='http://schemas.microsoft.com/winfx/2006/xaml/presentation'> ");
            sb.Append("<StackPanel xmlns='http://schemas.microsoft.com/winfx/2006/xaml/presentation' Orientation='Horizontal'> ");
            sb.Append("<Image Source='{Binding Path=" + _sImagePath + ", ElementName=" + _cpPane.Name + "}' /> ");
            sb.Append("<TextBlock Text='{Binding Path=" + _sTextPath + ", ElementName=" + _cpPane.Name + "}' /> ");
            sb.Append("</StackPanel> ");
            sb.Append("</DataTemplate> ");

            return XamlReader.Load(sb.ToString()) as DataTemplate;
        }
        private void evhxgFullRep_MouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {   // IG:95417 Make context menu visibible when right click on grid occurs until IG bug is fixed. 
            Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(sender as XamGrid);
            cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Visible;
        }
        #region Event Handlers
        public void vShowFullRepEx(object _sender, NotificationEventArgs<TestDescriptor> _evArgs)
        {
            // turn busy indicator back on
            m_vm.sBusyText = ViewStrings.GeneratingReport;
            m_vm.bIsBusy = true;
            if (_evArgs.Message != MessageTokens.sNO_DATA) 
                vShowFullRep(_evArgs.Data);
            // Launch any additional batch reports.
            if (iBatchNdx < iBatchCount)
            {
                xmiBatchFullRepEx();
            }
            else
            {
                m_vm.bIsBusy = false;
            }
        }

        private void vShowFullRep(TestDescriptor _oTestDescriptor) 
        {         
#if XDM_NAVIGATE_BY_INDEX  
            TabGroupPane tgpData = mytgpData;
#else
            TabGroupPane tgpData = dmPDB.FindPane("tgpData") as TabGroupPane;
#endif
            _bFullRepPaneReady = false;

            // Verify that we have not exceeded the conentpane limit.
            if (tgpData.Panes.Count > iCONTENT_PANE_LIMIT)
            {
                var mbResult = MessageBox.Show("You have reached tab limit of 10", "Confirmation", MessageBoxButton.OK);
                return;
            }

            // Create the content pane name based upon the test/row that was choosen.
            string sName = _oTestDescriptor.sFamilyId + "_" + _oTestDescriptor.sTestTime + "_" + _oTestDescriptor.sStandCode;
            // remove white spaces ( grid can't have name with spaces and we assign this to grid name too )
            sName = sName.Replace(" ", "_");
            sName = sName.Replace(":", "_");
            sName = sName.Replace("/", "_");

            // if the pane already exists, just activate and return
            if (boActivatePane(sName)) return;

            ContentPaneEx cpContentPaneEx = (ContentPaneEx)dmPDB.FindPane(sName);

            if (cpContentPaneEx == null)
            {
                cpContentPaneEx = vCreateContentPane(sName,
                                                    ApplicationStrings.SerialCode + ":" + _oTestDescriptor.sSerialCode + ", " + ApplicationStrings.StandCode + ":" + _oTestDescriptor.sStandCode,
                                                    _oTestDescriptor.sTestTime,
                                                    "/WMAApp;component/Assets/Images/Home/FullReport16x16.png",
                                                    "toolTipTemplate"
                                                    );
            }

            #region Grid
            // Create new grid to add to content pane.
            XamGrid xg = new XamGrid();
#if SUPPORT_FULL_REP_RESIZE
            xg.Name = "FR_" + sName; // +"X"; // add "X" to make xamgrid name different from the contentpaneex that encloses it.
            // This is necessary because contentpane name and grid name have to differ in visual tree.
#endif


            #region Columns
            xg.AutoGenerateColumns = false;
            // Hidden columns that can be selected and displayed
            xg.Columns.Add(oMakeTextColumn("iThreadNum", "", false));
            xg.Columns.Add(oMakeTextColumn("iPhaseOrderNum", "", false));
            xg.Columns.Add(oMakeTextColumn("iStepOrderNum", "", false));
            xg.Columns.Add(oMakeTextColumn("iPhaseId", ViewStrings.PhaseId, false));
            xg.Columns.Add(oMakeTextColumn("sPhase", ViewStrings.PhaseText, false));
            xg.Columns.Add(oMakeTextColumn("iStepId", ViewStrings.StepId, false));
            xg.Columns.Add(oMakeTextColumn("sStep", ViewStrings.StepText, false));
            // Visible Columns
            xg.Columns.Add(oMakeTextColumn("dId", ViewStrings.TestPointId));
            xg.Columns.Add(oMakeTextColumn("sText", ViewStrings.Text));

            xg.Columns.Add(oMakeTextColumn("dFormattedLowLimit", ViewStrings.LowLimit));
            xg.Columns.Add(oMakeTextColumn("dFormattedValue", ViewStrings.Value));
            xg.Columns.Add(oMakeTextColumn("dFormattedHighLimit", ViewStrings.HighLimit));
			xg.Columns.Add(oMakeTextColumn("sUnits", ViewStrings.Units));
           
            //// The remaining columns are hidden and allow me to know to know to which test I belong and test point version.
            //xg.Columns.Add(oMakeTextColumn("dtTime", "", false));
            //xg.Columns.Add(oMakeTextColumn("sStand", "", false));
            //xg.Columns.Add(oMakeTextColumn("sStand", "", false)); 

            #region ConditionalFormatting
            // conditional formatting
            // Precedence is given from top/first rule to bottom with the first rule given the top priority.
            // Setting the IsTerminalRule property on a condition means that when that condition is evaluated, the following conditions will not be evaluated.

            xg.ConditionalFormattingSettings.AllowConditionalFormatting = true;
            xg.Columns.DataColumns["dFormattedValue"].ConditionalFormatCollection.Add(new ValConditionalFormatRule());
            xg.Columns.DataColumns["dFormattedLowLimit"].ConditionalFormatCollection.Add(new LLConditionalFormatRule());
            xg.Columns.DataColumns["dFormattedHighLimit"].ConditionalFormatCollection.Add(new HLConditionalFormatRule());

            #endregion ConditionalFormatting 
            
            #endregion Columns
            #region Style
            xg.Margin = new Thickness(5);
            xg.HeaderStyle = (Style)this.Resources["styGridHeader"];
            xg.IsAlternateRowsEnabled = true;
            xg.CellStyle = (Style)this.Resources["styTestListGridCell"];
            xg.HorizontalAlignment = System.Windows.HorizontalAlignment.Stretch;
            xg.VerticalAlignment = System.Windows.VerticalAlignment.Top;
            xg.ColumnWidth = ColumnWidth.Star;
            #endregion Style 
           
            #region ColumnChooser
            xg.ColumnChooserSettings.AllowHiddenColumnIndicator = false;
            xg.ColumnChooserSettings.AllowHideColumnIcon = false;
            xg.ColumnChooserSettings.AllowColumnMoving = true;
            xg.ColumnChooserSettings.Style = (Style)this.Resources["styColumnChooser"];
            // Hide certain columns from column chooser

            #endregion ColumnChooser 
            
            #region ColumnMoving
            xg.ColumnMovingSettings.AllowColumnMoving = ColumnMovingType.Indicator;
            #endregion ColumnMoving
            #region ColumnFiltering
            xg.FilteringSettings.AllowFiltering = FilterUIType.FilterMenu;
            xg.FilteringSettings.FilteringScope = FilteringScope.ColumnLayout;
            #endregion ColumnFiltering
            #region ColumnFixing
            xg.FixedColumnSettings.AllowFixedColumns = FixedColumnType.DropArea;
            #endregion ColumnFixing
            #region ColumnSorting
            xg.SortingSettings.AllowMultipleColumnSorting = true;
            xg.SortingSettings.ShowSortIndicator = true;
            #endregion ColumnSorting
            #region Selection
            xg.SelectionSettings.RowSelection = SelectionType.Single;
            xg.SelectionSettings.CellClickAction = CellSelectionAction.SelectRow;
            xg.SelectionSettings.CellSelection = SelectionType.None;
            xg.RowSelectorSettings.EnableRowNumbering = false;
            xg.RowSelectorSettings.Visibility = System.Windows.Visibility.Collapsed;
            #endregion Selection
            #region GroupBy
            xg.GroupBySettings.AllowGroupByArea = GroupByAreaLocation.Top;
            xg.GroupByCollectionChanged += evhxgFullRep_GroupByChanged;
            #endregion GroupBy
            #region Summary
            xg.SummaryRowSettings.AllowSummaryRow = SummaryRowLocation.Bottom;
            #endregion Summary
           
            #region Scrolling
            xg.DeferredScrollingSettings.AllowDeferredScrolling = DeferredScrollingType.Default;
            xg.DeferredScrollingSettings.DeferredScrollTemplate = (DataTemplate)this.Resources["FullRepDeferredScrollingToolTip"];

            #endregion Scrolling 
             
            xg.ItemsSource = this.m_vm.ocTestData;

            #region Events
            // setup event handlers
#if SUPPORT_FULL_REP_RESIZE
            xg.InitializeRow += new EventHandler<InitializeRowEventArgs>(evhFullRep_InitializeRow);
#endif
            #if SUPPORT_CHART_LAUNCH_FROM_FUL_REP
            xg.CellDoubleClicked += new EventHandler<CellClickedEventArgs>(evhxgFullRep_CellDoubleClicked);
            //test xg.MouseRightButtonDown += new System.Windows.Input.MouseButtonEventHandler(xg_MouseRightButtonDown);
            #endif
            // IG:95417 Make context menu visibible when right click on grid occurs until IG bug is fixed.
            xg.MouseRightButtonDown += new System.Windows.Input.MouseButtonEventHandler(evhxgFullRep_MouseRightButtonDown);                                

            #endregion Events
             #region Menu
           
            Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);

            cmMgr = new ContextMenuManager();
            cmMgr.OpenMode = OpenMode.RightClick;
            Infragistics.Controls.Menus.ContextMenuService.SetManager(xg, cmMgr);
            cmMgr.ContextMenu = new XamContextMenu();
            cmMgr.ContextMenu.Placement = Infragistics.Controls.Menus.PlacementMode.Center;
            cmMgr.ContextMenu.Template = (ControlTemplate)this.Resources["ctContextMenuFullRep"];
            // evhContextMenu_Opening is specified in ctContextMenuFullRep xaml but not getting called so I hook it programatically
            cmMgr.ContextMenu.Opening += new EventHandler<OpeningEventArgs>(evhContextMenu_Opening);
            cmMgr.ContextMenu.IsOpen = false;

            #endregion Menu

            #endregion Grid
            // Add grid to content pane.
            cpContentPaneEx.Content = xg;

            // setup event handlers
#if SUPPORT_FULL_REP_RESIZE
            cpContentPaneEx.SizeChanged += new SizeChangedEventHandler(evhFullRepContentPane_SizeChanged);
#endif

            // Add new content pane to the tabbed group if pane wasn't already found.            
            if (tgpData.FindName(sName) == null) tgpData.Panes.Add(cpContentPaneEx);
            
        }

        #if SUPPORT_FULL_REP_RESIZE
        private void evhFullRep_InitializeRow(object _sender, InitializeRowEventArgs _evArgs)
        {
            // This check keeps the pane from performing these operations when a new row is added due
            // to virtualization.
            if (!_bFullRepPaneReady)
            {
                // It's ok to do this every frame since each subsequent row can only increase the column width.
                double totalWidth = 0;

                // Calculate the total width taken up by all the columns.
                foreach (Column col in (_sender as XamGrid).Columns)
                    totalWidth += col.ActualWidth;

                // Now calculate the percentages for each of the columns.
                foreach (Column col in (_sender as XamGrid).Columns)
                {
                    ColumnInfo cInfo = new ColumnInfo();
                    cInfo.dRelativeWidth = col.ActualWidth / totalWidth;
                    cInfo.dWidth = col.ActualWidth;

                    // Store it for later use.
                    col.Tag = cInfo;
                }

                // Use a dispatcher to delay setting the panes width.
                Dispatcher.BeginInvoke(() =>
                {
#if XDM_NAVIGATE_BY_INDEX
                    TabGroupPane tgpData = mytgpData;
#else
                    TabGroupPane tgpData = (Infragistics.Controls.Layouts.TabGroupPane)dmPDB.FindPane("tgpData");
#endif
                    if (tgpData != null)
                    {
                        ContentPaneEx cp = dmPDB.FindPane(tgpData.SelectedPane.Name) as ContentPaneEx;
                        if (cp != null)
                        {
                            //IProvideScrollInfo scrollInfo = cp.FindName("xgTestList") as IProvideScrollInfo;
                            //dScrollBarWidth = scrollInfo.VerticalScrollBar.ActualWidth + 20;
                            // Add some extra padding.
                            cp.FloatWidth = totalWidth + dScrollBarWidth;
                        }
                    }
                    // The order in which things occur is that this dispatcher code will be executed a number of times for each row initialized.
                    // This will also set calculating to false.  Once these initial rows are finished, the SizeChanged event on the pane will
                    // be executed.
                    _bFullRepStillCalculating = false;
                });
            }
        }
        private void evhFullRepContentPane_SizeChanged(object _sender, SizeChangedEventArgs _evArgs)
        {

            // As long as we're no longer calculating.
            if (!_bFullRepStillCalculating)
            {
                // If we get here that means all the initial rows have been created.
                _bFullRepPaneReady = true;

                // XamGrid xg = (_sender as ContentPaneEx).FindName(((ContentPaneEx)_sender).Name + "X") as XamGrid;
                XamGrid xg = (_sender as ContentPaneEx).FindName("FR_" + ((ContentPaneEx)_sender).Name) as XamGrid;

                if (xg != null) // todo: find out why this is null sometimes
                {
                    //// Ignore if this is a dummy pane...
                    //#if FIX_IG_92858
                    //if (xg.Name.StartsWith("cpDummy"))
                    //{
                    //    return;
                    //}
                    //#endif

                    // calculate widths with percentages so they keep the same relative size to each other.
#if SUPPORT_TEST_LIST_BANDS                    
                    foreach (object o in xg.Columns)
                    {
                        if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                        {
                            Column col = o as Column;
                            // if column was collapsed, ignore it
                            if (col.Visibility == System.Windows.Visibility.Visible)
                            {
                                if (col.Tag != null) // ask rob
                                {
                                    ColumnInfo cInfo = col.Tag as ColumnInfo;
                                    int pixels = (int)(cInfo.dRelativeWidth * (_evArgs.NewSize.Width - dScrollBarWidth));
                                    col.Width = new ColumnWidth(pixels, false);
                                }
                            }
                        }
                    }
#else
                    foreach (Column col in xg.Columns)
                    {
                        if (col.Tag != null) // ask rob
                        {
                            ColumnInfo cInfo = col.Tag as ColumnInfo;
                            int pixels = (int)(cInfo.dRelativeWidth * (_evArgs.NewSize.Width - dScrollBarWidth));
                            col.Width = new ColumnWidth(pixels, false);
                        }
                    }
#endif
                }
            }
        }
        #endif // SUPPORT_FULL_REP_RESIZE

        #if SUPPORT_CHART_LAUNCH_FROM_FUL_REP
        void evhxgFullRep_CellDoubleClicked(object _sender, CellClickedEventArgs _evArgs)
        {
            if (_evArgs.Cell.Row.Data.GetType() != typeof(TestData)) return;
            iBatchNdx = iBatchCount = 0; // we are not doing batch style
            TestData td = _evArgs.Cell.Row.Data as TestData;
            m_vm.vGetTPData(new TPDescriptor(td.dTpDefnSakey, td.dId, td.sText, td.sUnits));
        }
      
        #endif

        #endregion Event Handlers

        #region ConditionalFormatting
        #region LL
        public class LLConditionalFormatRuleProxy : ConditionalFormattingRuleBaseProxy
        {
            private Style styBlackOnYellow { get; set; }
            private Style styWhiteOnRed { get; set; }

            public LLConditionalFormatRuleProxy()
            {
                Style styWhiteOnRed = new Style(typeof(ConditionalFormattingCellControl));
                styWhiteOnRed.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.BackgroundProperty, Value = new SolidColorBrush(Colors.Red) });
                styWhiteOnRed.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.AltBackgroundProperty, Value = new SolidColorBrush(Colors.Red) });
                styWhiteOnRed.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.ForegroundProperty, Value = new SolidColorBrush(Colors.White) });
                this.styWhiteOnRed = styWhiteOnRed;

                Style styBlackOnYellow = new Style(typeof(ConditionalFormattingCellControl));
                styBlackOnYellow.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.BackgroundProperty, Value = new SolidColorBrush(Colors.Yellow) });
                styBlackOnYellow.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.AltBackgroundProperty, Value = new SolidColorBrush(Colors.Yellow) });
                styBlackOnYellow.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.ForegroundProperty, Value = new SolidColorBrush(Colors.Black) });
                this.styBlackOnYellow = styBlackOnYellow;

                this.styBlackOnYellow = styBlackOnYellow;
            }

            protected override Style EvaluateCondition(object _sourceDataObject, object _sourceDataValue)
            {
                TestData d = (TestData)_sourceDataObject;

                //    'TP_QTY  & LOW_LIMIT
                //    If (Not LL Is DBNull.Value) Then
                //        If (TpQty < Convert.ToDouble(LL)) Then
                //            ' white on red
                //            TpQtyCSS = "WMA_CSSC_WhiteOnRed"
                //            items(cols("FORMATTED_LOW_LIMIT").Index).CssClass = "WMA_CSSC_WhiteOnRed"
                //        End If
                //        If ((TpQty >= Convert.ToDouble(LL)) And (TpQty < (Convert.ToDouble(LL) + System.Math.Abs(Convert.ToDouble(HL) - Convert.ToDouble(LL)) * 0.1))) Then
                //            TpQtyCSS = "WMA_CSSC_BlackOnYellow"
                //            items(cols("FORMATTED_LOW_LIMIT").Index).CssClass = "WMA_CSSC_BlackOnYellow"
                //        End If
                //    End If

                //    'TP_QTY  & HIGH_LIMIT
                //    If (Not HL Is DBNull.Value) Then
                //        If (TpQty > Convert.ToDouble(HL)) Then
                //            ' white on red
                //            TpQtyCSS = "WMA_CSSC_WhiteOnRed"
                //            items(cols("FORMATTED_HIGH_LIMIT").Index).CssClass = "WMA_CSSC_WhiteOnRed"
                //        End If
                //        If ((TpQty <= Convert.ToDouble(HL)) And (TpQty > (Convert.ToDouble(HL) - System.Math.Abs(Convert.ToDouble(HL) - Convert.ToDouble(LL)) * 0.1))) Then
                //            TpQtyCSS = "WMA_CSSC_BlackOnYellow"
                //            items(cols("FORMATTED_HIGH_LIMIT").Index).CssClass = "WMA_CSSC_BlackOnYellow"
                //        End If
                //    End If

                if (d.dFormattedLowLimit != null)
                {
                    if (d.dFormattedValue < d.dFormattedLowLimit)
                    {
                        return this.styWhiteOnRed;
                    }
                    if (Convert.ToDouble(d.dFormattedValue) <
                        (Convert.ToDouble(d.dFormattedLowLimit) + System.Math.Abs(Convert.ToDouble(d.dFormattedHighLimit) - Convert.ToDouble(d.dFormattedLowLimit)) * 0.1))
                    {
                        return this.styBlackOnYellow;
                    }
                }

                return null;  // leave it unstyled;
            }
        }
        public class LLConditionalFormatRule : ConditionalFormattingRuleBase
        {
            protected override IConditionalFormattingRuleProxy CreateProxy()
            {
                return new LLConditionalFormatRuleProxy();
            }
        }
        #endregion LL
        #region HL
        public class HLConditionalFormatRuleProxy : ConditionalFormattingRuleBaseProxy
        {
            private Style styBlackOnYellow { get; set; }
            private Style styWhiteOnRed { get; set; }

            public HLConditionalFormatRuleProxy()
            {
                Style styWhiteOnRed = new Style(typeof(ConditionalFormattingCellControl));
                styWhiteOnRed.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.BackgroundProperty, Value = new SolidColorBrush(Colors.Red) });
                styWhiteOnRed.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.AltBackgroundProperty, Value = new SolidColorBrush(Colors.Red) });
                styWhiteOnRed.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.ForegroundProperty, Value = new SolidColorBrush(Colors.White) });
                this.styWhiteOnRed = styWhiteOnRed;

                Style styBlackOnYellow = new Style(typeof(ConditionalFormattingCellControl));
                styBlackOnYellow.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.BackgroundProperty, Value = new SolidColorBrush(Colors.Yellow) });
                styBlackOnYellow.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.AltBackgroundProperty, Value = new SolidColorBrush(Colors.Yellow) });
                styBlackOnYellow.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.ForegroundProperty, Value = new SolidColorBrush(Colors.Black) });
                this.styBlackOnYellow = styBlackOnYellow;

                this.styBlackOnYellow = styBlackOnYellow;
            }

            protected override Style EvaluateCondition(object _sourceDataObject, object _sourceDataValue)
            {
                TestData d = (TestData)_sourceDataObject;

                if (d.dFormattedHighLimit != null)
                {
                    if (d.dFormattedValue > d.dFormattedHighLimit)
                    {
                        return this.styWhiteOnRed;
                    }
                    if (Convert.ToDouble(d.dFormattedValue) >
                        (Convert.ToDouble(d.dFormattedHighLimit) - System.Math.Abs(Convert.ToDouble(d.dFormattedHighLimit) - Convert.ToDouble(d.dFormattedLowLimit)) * 0.1))
                    {
                        return this.styBlackOnYellow;
                    }
                }
                return null;  // leave it unstyled;
            }
        }
        public class HLConditionalFormatRule : ConditionalFormattingRuleBase
        {
            protected override IConditionalFormattingRuleProxy CreateProxy()
            {
                return new HLConditionalFormatRuleProxy();
            }
        }
        #endregion HL
        #region Val
        public class ValConditionalFormatRuleProxy : ConditionalFormattingRuleBaseProxy
        {
            private Style styBlackOnYellow { get; set; }
            private Style styWhiteOnRed { get; set; }

            public ValConditionalFormatRuleProxy()
            {
                Style styWhiteOnRed = new Style(typeof(ConditionalFormattingCellControl));
                styWhiteOnRed.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.BackgroundProperty, Value = new SolidColorBrush(Colors.Red) });
                styWhiteOnRed.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.AltBackgroundProperty, Value = new SolidColorBrush(Colors.Red) });
                styWhiteOnRed.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.ForegroundProperty, Value = new SolidColorBrush(Colors.White) });
                this.styWhiteOnRed = styWhiteOnRed;

                Style styBlackOnYellow = new Style(typeof(ConditionalFormattingCellControl));
                styBlackOnYellow.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.BackgroundProperty, Value = new SolidColorBrush(Colors.Yellow) });
                styBlackOnYellow.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.AltBackgroundProperty, Value = new SolidColorBrush(Colors.Yellow) });
                styBlackOnYellow.Setters.Add(new Setter() { Property = ConditionalFormattingCellControl.ForegroundProperty, Value = new SolidColorBrush(Colors.Black) });
                this.styBlackOnYellow = styBlackOnYellow;

                this.styBlackOnYellow = styBlackOnYellow;
            }

            protected override Style EvaluateCondition(object _sourceDataObject, object _sourceDataValue)
            {
                TestData d = (TestData)_sourceDataObject;
                if (d.dFormattedLowLimit != null)
                {
                    if (d.dFormattedValue < d.dFormattedLowLimit)
                    {
                        return this.styWhiteOnRed;
                    }
                    if (Convert.ToDouble(d.dFormattedValue) <
                        (Convert.ToDouble(d.dFormattedLowLimit) + System.Math.Abs(Convert.ToDouble(d.dFormattedHighLimit) - Convert.ToDouble(d.dFormattedLowLimit)) * 0.1))
                    {
                        return styBlackOnYellow;
                    }
                }
                if (d.dFormattedHighLimit != null)
                {
                    if (d.dFormattedValue > d.dFormattedHighLimit)
                    {
                        return this.styWhiteOnRed;
                    }
                    if (Convert.ToDouble(d.dFormattedValue) >
                        (Convert.ToDouble(d.dFormattedHighLimit) - System.Math.Abs(Convert.ToDouble(d.dFormattedHighLimit) - Convert.ToDouble(d.dFormattedLowLimit)) * 0.1))
                    {
                        return this.styBlackOnYellow;
                    }
                }
                return null;  // leave it unstyled;
            }
        }
        public class ValConditionalFormatRule : ConditionalFormattingRuleBase
        {
            protected override IConditionalFormattingRuleProxy CreateProxy()
            {
                return new ValConditionalFormatRuleProxy();
            }
        }
        #endregion Val
        #endregion ConditionalFormatting
        
        // utility method
        private TextColumn oMakeTextColumn(string _sKey, string _sHeaderText, bool _bVisible = true)
        {
            return new TextColumn()
            {
                Key = _sKey,
                HeaderText = _sHeaderText,
                //DataType = rrp ??? 
                AllowCaseSensitiveSort = true,
                IsFilterable = true,
                IsFixable = true,
                IsGroupable = true,
                IsHideable = true,
                IsMovable = true,
                IsResizable = true,
                IsSortable = true,
                Visibility = (_bVisible ? System.Windows.Visibility.Visible : System.Windows.Visibility.Collapsed),
                IsSummable = (_sKey == "dId" ? true : false), // for debug purposes, allow summing on dId
                VerticalContentAlignment = System.Windows.VerticalAlignment.Center,
                IsReadOnly = true,
                AllowToolTips = AllowToolTips.Always
                //ToolTipStyle= new Style("ToolTip"){ Setters = new Setter("Foreground,"Red")}
            };
        }
        #region Grouping
        private void evhxgFullRep_GroupByChanged(object _sender, Infragistics.Controls.Grids.GroupByCollectionChangedEventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)_sender;
            // If were are grouping, then don't show the columns that are grouped in the grid
            if (xg.GroupBySettings.GroupByOperation == GroupByOperation.GroupByRows)
            {
            #if SUPPORT_TEST_LIST_BANDS                    
                foreach (object o in xg.Columns)
                {
                    if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                    {
                        Column c = o as Column;
                        // if column was collapsed, ignore it
                        if (c.Visibility == System.Windows.Visibility.Visible)
                        {
                            if (_evArgs.NewGroupedColumns.Contains(c))
                            {
                                c.Visibility = System.Windows.Visibility.Collapsed;
                            }
                            else
                            {
                                c.Visibility = System.Windows.Visibility.Visible;
                            }
                        }
                    }
                }
            #else
                foreach (Infragistics.Controls.Grids.Column c in xg.Columns)
                {
                    if (_evArgs.NewGroupedColumns.Contains(c))
                    {
                        c.Visibility = System.Windows.Visibility.Collapsed;
                    }
                    else
                    {
                        c.Visibility = System.Windows.Visibility.Visible;
                    }
                }
#endif
            }
        }
        private void xmiFullRepShowGrouped_Click(object _sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {

                //todo: do I group by phase/step order num or just phase/step id???

                //xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iThreadNum"]);
                //xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iPhaseOrderNum"]);
                //xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iStepOrderNum"]);
                xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iThreadNum"]);
                xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iPhaseId"]);
                xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sPhase"]);
                xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iStepId"]);
                xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sStep"]);
                xg.GroupBySettings.GroupByOperation = GroupByOperation.GroupByRows;

                // close the context menu. Why doesn't this close automatically????
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
        private void xmiFullRepShowMerged_Click(object _sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {

                //todo: do I group by phase/step order num or just phase/step id???

                //xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iThreadNum"]);
                //xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iPhaseOrderNum"]);
                //xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iStepOrderNum"]);
                Dispatcher.BeginInvoke(() =>
                {
                    xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iThreadNum"]);
                    xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iPhaseId"]);
                    xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sPhase"]);
                    xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["iStepId"]);
                    xg.GroupBySettings.GroupByColumns.Add(xg.Columns.DataColumns["sStep"]);

#if SUPPORT_TEST_LIST_BANDS
                    foreach (object o in xg.Columns)
                    {
                        if (o.GetType() != typeof(Infragistics.Controls.Grids.ColumnLayout))
                        {
                            Column c = o as Column;
                            c.Visibility = System.Windows.Visibility.Visible;
                        }
                    }
#else

                        foreach (Infragistics.Controls.Grids.Column c in xg.Columns)
                        {
                            c.Visibility = System.Windows.Visibility.Visible;
                        }
#endif
                    xg.GroupBySettings.GroupByOperation = GroupByOperation.MergeCells;
                });


                // close the context menu. Why doesn't this close automatically????
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
        #endregion Grouping
        #region Context Menu
        private void cmnuFullRep_Item_Clicked(object _sender, ItemClickedEventArgs _evArgs)
        {
            // menu level click handler
            // IG:95417 Make context menu invisibible when item is choosen until IG bug is fixed. 
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
        #endregion Context Menu

        #endregion Full Rep        
        #region TP Data

        public ContentPaneEx FindPane(string sName)
        {
            ContentPaneEx cpex = null;
            // panes and closedPane collection. 
            cpex = dmPDB.FindPane(sName) as ContentPaneEx;
            return cpex;
        }

        public void vShowTpLimitsEx(object _sender, NotificationEventArgs<TPDescriptor> _evArgs)
        {
            if (_evArgs.Message != MessageTokens.sNO_DATA)
            {
                m_vm.sBusyText = ViewStrings.GeneratingChart;
                m_vm.bIsBusy = true;

                if ( _evArgs.Data.iChartType == 1)
                {
                    // get the content pane name based upon the test point that was choosen.
                    ContentPaneEx cpex = FindPane(_evArgs.Data.dId.ToString() + "Trend");

                    if (cpex != null)
                    {
                        ucXdcTrendChart ucXdc = (ucXdcTrendChart)cpex.Content;

                        if (_evArgs.Data.sLimitType == "L")
                        {
                            ucXdc.vAddLimitData(_evArgs.Data.sLimitType, m_vm.ocLowLimit);
                        }
                        else
                        {
                            ucXdc.vAddLimitData(_evArgs.Data.sLimitType, m_vm.ocHighLimit);
                        }

                        cpex.Content = ucXdc;
                    }
                }
                else
                {
                    // get the content pane name based upon the test point that was choosen.
                    ContentPaneEx cpex = FindPane(_evArgs.Data.dId.ToString() + "Histogram");

                    if (cpex != null)
                    {
                        ucHistogramChart ucXdx = (ucHistogramChart)cpex.Content;

                        cpex.Content = ucXdx;
                    }

                }

                m_vm.bIsBusy = false;
            }
        }

        public void vShowTPDataEx(object _sender, NotificationEventArgs<TPDescriptor> _evArgs)
        {
            // turn busy indicator back on
            m_vm.sBusyText = ViewStrings.GeneratingChart;
            m_vm.bIsBusy = true;
            if (_evArgs.Message != MessageTokens.sNO_DATA) 
                vShowTPData(_evArgs.Data);
            // Launch any additional batch reports.
            if (iBatchNdx < iBatchCount)
            {
                xmiBatchTPDataEx(1);
            }
            else
            {
                m_vm.bIsBusy = false;
            }
        }

        private void vShowTPData(TPDescriptor _oTestDescriptor)
        {
            if (mytgpData != null)
            {
                if (mytgpData.Panes.Count > iCONTENT_PANE_LIMIT)
                {
                    var mbResult = MessageBox.Show("You have reached tab limit of 10", "Confirmation", MessageBoxButton.OK);
                    return;
                }          
                
                #region Chart
                    
                // add control to content pane

                if (_oTestDescriptor.iChartType == 1)
                {
                    // Create the content pane name based upon the test point that was choosen.
                    string sName = _oTestDescriptor.dId.ToString() + "Trend";

                    // if the pane already exists, just activate and return
                    if (boActivatePane(sName))
                    {
                        return;
                    }

                    ContentPaneEx cpContentPaneEx = (ContentPaneEx)dmPDB.FindPane(sName);

                    if (cpContentPaneEx == null)
                    {
                        cpContentPaneEx = vCreateContentPane(sName,
                                                                  ViewStrings.TestPointId + ": " + _oTestDescriptor.dId.ToString() + "-Trend",
                                                                  _oTestDescriptor.dId.ToString(),
                                                                  "/WMAApp;component/Assets/Images/Home/LineChart16x16.png",
                                                                  "toolTipTemplate"
                                                                  );
                    }

                    ucXdcTrendChart ucXdc = new ucXdcTrendChart();
                    cpContentPaneEx.Content = ucXdc;

                    // Make sure we setup the chart, default type is Trend
                    ucXdc.vSetupChart();
                    
                    ucXdc.vSetTitle( ViewStrings.TrendChartTitle + ":" + 
                                     _oTestDescriptor.dId.ToString() + " " + 
                                     _oTestDescriptor.sText + " - " + 
                                     _oTestDescriptor.sUnits );
                    
                    ucXdc.vSetData(m_vm.ocTPData);

                    // Add new content pane to the tabbed group if pane wasn't already found.            
                    if (mytgpData.FindName(sName) == null) mytgpData.Panes.Add(cpContentPaneEx);
                }
                else if (_oTestDescriptor.iChartType == 2)
                {
                    // Create the content pane name based upon the test point that was choosen.
                    string sName = _oTestDescriptor.dId.ToString() + "Histogram";

                    // if the pane already exists, just activate and return
                    if (boActivatePane(sName))
                    {
                        return;
                    }

                    ContentPaneEx cpContentPaneEx = (ContentPaneEx)dmPDB.FindPane(sName);

                    if (cpContentPaneEx == null)
                    {
                        cpContentPaneEx = vCreateContentPane(sName,
                                                                  ViewStrings.TestPointId + ": " + _oTestDescriptor.dId.ToString() + "-Histo",
                                                                  _oTestDescriptor.dId.ToString(),
                                                                  "/WMAApp;component/Assets/Images/Home/LineChart16x16.png",
                                                                  "toolTipTemplate"
                                                                  );
                    }

                    ucHistogramChart ucXdc = new ucHistogramChart();
                    cpContentPaneEx.Content = ucXdc;

                    ucXdc.vSetupChart(ViewModels.Charts.ChartTypes.ctHistogram);
                    
                    ucXdc.vSetTitle( ViewStrings.HistogramChartTitle + ":" + 
                                     _oTestDescriptor.dId.ToString() + " " + 
                                     _oTestDescriptor.sText + " - " +
                                     _oTestDescriptor.sUnits );

                    ucXdc.vSetData(m_vm.ocTPData);

                    ucXdc.vComputeChartMetrics("S");

                    // Add new content pane to the tabbed group if pane wasn't already found.            
                    if (mytgpData.FindName(sName) == null) mytgpData.Panes.Add(cpContentPaneEx);
                }

                #endregion Chart
            }
        }
        
        #endregion TP Data
        #region TP Pivot Data
        public void vShowTPPivotGridDataEx(object _sender, NotificationEventArgs<TPDescriptor> _evArgs)
        {
            // turn busy indicator back on
            m_vm.sBusyText = ViewStrings.GeneratingReport;
            m_vm.bIsBusy = true;

            if (_evArgs.Message != MessageTokens.sNO_DATA)
                vShowTPPivotGridData(_evArgs.Data);
            
            m_vm.bIsBusy = false;
        }
        private void vShowTPPivotGridData(TPDescriptor _oTestDescriptor)
        {
            #region pane size

#if XDM_NAVIGATE_BY_INDEX
            TabGroupPane tgpData = mytgpData;
#else
            TabGroupPane tgpData = dmPDB.FindPane("tgpData") as TabGroupPane;
#endif
            #endregion pane size

            // if the pane already exists, remove it and rebuild
            if (boActivatePane("cpPivotGrid"))
            {
               // tgpData.Panes.Remove((ContentPaneEx)tgpData.FindName("cpPivotGrid"));
            }

            ContentPaneEx cpContentPaneEx = (ContentPaneEx)tgpData.FindName("cpPivotGrid");

            if (cpContentPaneEx == null)
            {
                cpContentPaneEx = vCreateContentPane("cpPivotGrid",
                                                        "Pivot Grid Data",
                                                        "Pivot Grid",
                                                        "/WMAApp;component/Assets/Images/Home/LineChart16x16.png",
                                                        "toolTipTemplate"
                                                        );
            }

            if (cpContentPaneEx.Content == null)
            {
                Grid gr = new Grid();

                gr.ColumnDefinitions.Add(new ColumnDefinition());
                gr.ColumnDefinitions.Add(new ColumnDefinition());
                gr.ColumnDefinitions[1].Width = new GridLength(1, GridUnitType.Auto);

                gr.RowDefinitions.Add(new RowDefinition());
                gr.RowDefinitions[0].Height = new GridLength(1, GridUnitType.Star);
                gr.RowDefinitions.Add(new RowDefinition());
                gr.RowDefinitions[1].Height = new GridLength(55, GridUnitType.Pixel);

                XamPivotGrid pg = new XamPivotGrid();
                pg.Name = "xpgPivotGrid";
                pg.ParentInFrontForColumns = true;
                pg.AllowCompactLayout = true;
                pg.MouseRightButtonDown += evhxgTPPivotGrid_MouseRightButtonDown;
                Grid.SetColumn(pg, 0);
                Grid.SetRow(pg, 0);

                gr.Children.Add(pg);

                Infragistics.Controls.Grids.Expander ex = new Infragistics.Controls.Grids.Expander();
                ex.Name = "expPivotGrid";
                ex.IsExpanded = false;
                Grid.SetRow(ex, 0);
                Grid.SetColumn(ex, 1);

                XamPivotDataSelector pds = new XamPivotDataSelector();
                pds.Name = "xpdsPivotDataSelector";
                ex.Content = pds;

                gr.Children.Add(ex);

                cpContentPaneEx.Content = gr;

                if (tgpData.FindName("cpPivotGrid") == null) tgpData.Panes.Add(cpContentPaneEx);
            }

            vBuildPivotGrid();
        }

        private void vBuildPivotGrid()
        {
            #region Data source
            // [metadata.displayname].[hierarchydescriptio.Hiearchyname] for rows and columns, and just the datasource property name for measures.
            FlatDataSource fdsData = new FlatDataSource
            {
                DimensionsGenerationMode = DimensionsGenerationMode.Metadata, // Basically, only show metadata information.
                ItemsSource = m_vm.ocTPPivotGridData,
                Cube = DataSourceBase.GenerateInitialCube("TPPivotGridData"), 
                //Rows = DataSourceBase.GenerateInitialItems("[dId].[dId]"),
                //Rows = DataSourceBase.GenerateInitialItems("[" + ViewStrings.IdMetadata + "].[" + ViewStrings.IdHierarchy + "]"),
                Rows = DataSourceBase.GenerateInitialItems("[" + ViewStrings.Dimensions + "].[" + ViewStrings.Id + "]"),
                //??Columns = DataSourceBase.GenerateInitialItems("[dtTestTime].[dtTestTime],[sStandCode].[sStandCode],[sModel].[sModel],[sSerial].[sSerial]"),
                //Columns = DataSourceBase.GenerateInitialItems("[" + ViewStrings.TestTimeMetadata + "].[" + ViewStrings.TestTimeHierarchy + "],[sStandCode].[sStandCode],[sModel].[sModel],[sSerial].[sSerial]"),
                Columns = DataSourceBase.GenerateInitialItems("[" + ViewStrings.Dimensions + "].[" + ViewStrings.TT_TestTime + "], " +
                                                              "[" + ViewStrings.Dimensions + "].[" + ViewStrings.Stand + "], " +
                                                              "[" + ViewStrings.Dimensions + "].[" + ViewStrings.Model + "], " +
                                                              "[" + ViewStrings.Dimensions + "].[" + ViewStrings.Serial + "]"),                
                Measures = DataSourceBase.GenerateInitialItems("dTPQty")
            };
            
            #endregion Data source

            #region Cube Settings
            // CubeMetadata class allows users to control names and appearance of dimensions in the dataselector tree.
            // The DisplayName is used to change the name of the hierarchy/measure in the UI and the DisplayFormat is used for measures. 
            // Raw value goes to the Value property of the cell and the formatted value goes to the FormattedValue property. 
            // The format in the string will show numbers as dollars (or other currency depending on the system culture) with double precision.

            #region Cube metadata
            CubeMetadata mdCube = new CubeMetadata()
            {
                DisplayName = ViewStrings.PivotGrid,
                DataTypeFullName = typeof(TPPivotGridData).FullName
            };
            fdsData.CubesSettings.Add(mdCube);

            #endregion Cube metadata     

            #region My dimensions
            // create a metadata node called "Dimensions" under which I will put Id, stand, model, serial, test time
            DimensionMetadata mdDimensions = new DimensionMetadata();
            mdDimensions.SourcePropertyName = "dId"; // Note: the IG framework syntax requires each DimensionMetadata have a source property
                                                     // but it will not be used so just pick any property in my data
            mdDimensions.DisplayName = ViewStrings.Dimensions;
            mdDimensions.DimensionType = Infragistics.Olap.FlatData.DimensionType.Dimension;
            mdCube.DimensionSettings.Add(mdDimensions);

            #region Id
            HierarchyDescriptor hdId = new HierarchyDescriptor();
            hdId.HierarchyName = ViewStrings.Id;
            hdId.SourcePropertyName = "dId";

            // Add the hierarchy descriptor for dId so it points to the right data.
            Expression<Func<TPPivotGridData, decimal>> expId = s => s.dId;
            hdId.LevelDescriptors.Add(new HierarchyLevelDescriptor(expId, ViewStrings.Id));
            mdDimensions.HierarchyDescriptors.Add(hdId);
            #endregion Id
            #region TestTime

            HierarchyDescriptor hdDateTime = new HierarchyDescriptor();// { AppliesToPropertiesOfType = typeof(DateTime) };
            hdDateTime.HierarchyName = ViewStrings.TT_TestTime;
            hdDateTime.SourcePropertyName = "dtTestTime";

            //hdDateTime.AddLevel<DateTime>(dt => SR.GetString("AllPeriods"), "All Periods");
            //hdDateTime.AddLevel<DateTime>(dt => dt.Year, "Years");
            //hdDateTime.AddLevel<DateTime>(dt => dt.MonthShort(), "Months");
            //hdDateTime.AddLevel<DateTime>(dt => dt.Day, "Days");

            // show test date and time on 2 lines in header so columns are not so wide
            Expression<Func<TPPivotGridData, string>> TimeExpression = t => t.dtTestTime.ToShortDateString().ToString() + System.Environment.NewLine + t.dtTestTime.TimeOfDay.ToString();
            hdDateTime.LevelDescriptors.Add(new HierarchyLevelDescriptor(TimeExpression, ViewStrings.TT_TestTime));
            mdDimensions.HierarchyDescriptors.Add(hdDateTime);

            #endregion TestTime
            #region Stand
            HierarchyDescriptor hdStand = new HierarchyDescriptor();
            hdStand.HierarchyName = ViewStrings.Stand;
            hdStand.SourcePropertyName = "sStand";

            // Add the hierarchy descriptor for dId so it points to the right data.
            Expression<Func<TPPivotGridData, string>> expStand = s => s.sStandCode;
            hdStand.LevelDescriptors.Add(new HierarchyLevelDescriptor(expStand, ViewStrings.Stand));
            mdDimensions.HierarchyDescriptors.Add(hdStand);
            #endregion Stand
            #region Model
            HierarchyDescriptor hdModel = new HierarchyDescriptor();
            hdModel.HierarchyName = ViewStrings.Model;
            hdModel.SourcePropertyName = "sModel";

            // Add the hierarchy descriptor for dId so it points to the right data.
            Expression<Func<TPPivotGridData, string>> expModel = s => s.sModel;
            hdModel.LevelDescriptors.Add(new HierarchyLevelDescriptor(expModel, ViewStrings.Model));
            mdDimensions.HierarchyDescriptors.Add(hdModel);
            #endregion Model
            #region Serial
            HierarchyDescriptor hdSerial = new HierarchyDescriptor();
            hdSerial.HierarchyName = ViewStrings.Serial;
            hdSerial.SourcePropertyName = "sSerial";

            // Add the hierarchy descriptor for dId so it points to the right data.
            Expression<Func<TPPivotGridData, string>> expSerial = s => s.sSerial;
            hdSerial.LevelDescriptors.Add(new HierarchyLevelDescriptor(expSerial, ViewStrings.Serial));
            mdDimensions.HierarchyDescriptors.Add(hdSerial);
            #endregion Serial

            #endregion My dimensions

            #region Measures
            #region TpQty
            DimensionMetadata mdTpQty = new DimensionMetadata();
            mdTpQty.SourcePropertyName = "dTPQty";
            mdTpQty.DisplayName = ViewStrings.Value; //ViewStrings.ValueMetadata;
            mdTpQty.DimensionType = Infragistics.Olap.FlatData.DimensionType.Measure; 
            mdCube.DimensionSettings.Add(mdTpQty);

            HierarchyDescriptor hdTpQty = new HierarchyDescriptor();
            hdTpQty.HierarchyName = ViewStrings.Value;
            hdTpQty.SourcePropertyName = "dTPQty";         

            // Add the hierarchy descriptor for TpQty so it points to the right data.
            Expression<Func<TPPivotGridData, decimal>> expTpQty = s => s.dTPQty;
            hdTpQty.LevelDescriptors.Add(new HierarchyLevelDescriptor(expTpQty, ViewStrings.Value));
            mdTpQty.HierarchyDescriptors.Add(hdTpQty);

            #endregion TpQty
            #endregion Measures

            #endregion Cube Settings
#if XDM_NAVIGATE_BY_INDEX
            TabGroupPane tgpPane = mytgpData;
            XamPivotGrid xpg = (XamPivotGrid)tgpPane.FindName("xpgPivotGrid");
            XamPivotDataSelector xpds = (XamPivotDataSelector)tgpPane.FindName("xpdsPivotDataSelector");

            if (xpg == null)
            {
                ContentPaneEx cp = (ContentPaneEx)tgpPane.FindName("cpPivotGrid");
                Grid gr = (Grid)cp.Content;
                xpg = (XamPivotGrid)gr.Children[0];

                Infragistics.Controls.Grids.Expander ex = (Infragistics.Controls.Grids.Expander)gr.Children[1];

                xpds = (XamPivotDataSelector)ex.Content;
            }
#else
            TabGroupPane tgpPane = (TabGroupPane)this.grDataArea.FindName("tgpData");
            XamPivotGrid xpg = (XamPivotGrid)tgpPane.FindName("xpgPivotGrid");
#endif
            xpg.DataSource = fdsData;
            xpds.DataSource = fdsData;
        }

        private void evhxgTPPivotGrid_MouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {   // IG:95417 Make context menu visibible when right click on grid occurs until IG bug is fixed. 
            Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(sender as XamPivotGrid);
            cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Visible;
        }
        #endregion TP Pivot Data
        #endregion Data
        #region XamGrid General
        //// Pertains to XamGrids in general
        private void xmiShowColumnChooser_Click(object _sender, EventArgs _evArgs)
        {
            //Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
            //Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName(sContextMenusXamGrid);
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                xg.ShowColumnChooser();           
                // close the context menu. Why doesn't this close automatically????
                //ig case 95417 Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                //ig case 95417 cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
        private void xmiGridReset_Click(object _sender, EventArgs _evArgs)
        {
            //Infragistics.Controls.Layouts.TabGroupPane tgpPane = (Infragistics.Controls.Layouts.TabGroupPane)this.grDataArea.FindName("tgpList");
            //Infragistics.Controls.Grids.XamGrid xg = (Infragistics.Controls.Grids.XamGrid)tgpPane.FindName("xgTestList");

            XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {
                xg.FilteringSettings.RowFiltersCollection.Clear();
                xg.SortingSettings.SortedColumns.Clear();
                xg.GroupBySettings.GroupByColumns.Clear();
#if SUPPORT_TEST_LIST_BANDS                    
                foreach (object o in xg.Columns)
                {
                    if (o.GetType() != typeof(ColumnLayout))
                    {
                        Column col = o as Column;
                        col.Visibility = Visibility.Visible;
                    }
                }
#else
                foreach (Column col in xg.Columns)
                {
                    col.Visibility = Visibility.Visible;
                }
#endif
               
                foreach (Row r in xg.Rows)
                {
                    r.IsExpanded = false;
                }


                //todo: how to reset columnMovingSettings to put columns in original order????
                //xg.ColumnMovingSettings.?????

                // close the context menu. Why doesn't this close automatically????
                //? Infragistics.Controls.Menus.ContextMenuManager cmMgr = Infragistics.Controls.Menus.ContextMenuService.GetManager(xg);
                //? cmMgr.ContextMenu.Visibility = System.Windows.Visibility.Collapsed;
            }
        }
        private void AutoExpandAllChildRows(Row _parentRow)
        {
            foreach (ChildBand cb in _parentRow.ChildBands)
            {
                cb.IsExpanded = true;
                if (cb.HasChildren)
                    AutoExpandAllChildBands(cb);
            }
        }
        private void AutoCollapseAllChildRows(Row _parentRow)
        {
            foreach (ChildBand cb in _parentRow.ChildBands)
            {
                cb.IsExpanded = false;
                if (cb.HasChildren)
                    AutoCollapseAllChildBands(cb);
            }
        }
        private void AutoExpandAllChildBands(ChildBand _parentBand)
        {
            foreach (Row r in _parentBand.Rows)
            {
                r.IsExpanded = true;
                if (r.HasChildren)
                    AutoExpandAllChildRows(r);
            }
        }
        private void AutoCollapseAllChildBands(ChildBand _parentBand)
        {
            foreach (Row r in _parentBand.Rows)
            {
                r.IsExpanded = false;
                if (r.HasChildren)
                    AutoCollapseAllChildRows(r);
            }
        }
        // ...
        private bool bAutoExpandingOrCollapsing = false;
        private void evhxgTestList_RowExpansionChanged(object sender, RowExpansionChangedEventArgs e)
        {
            if (!bAutoExpandingOrCollapsing)
            {
                // expand child rows only if parent row is expanded
                if (e.Row.IsExpanded)
                {
                    bAutoExpandingOrCollapsing = true;

                    if (e.Row.RowType == Infragistics.Controls.Grids.RowType.DataRow)
                        AutoExpandAllChildRows((Row)e.Row);

                    if (e.Row.RowType == Infragistics.Controls.Grids.RowType.ColumnLayoutHeaderRow)
                        AutoExpandAllChildBands((ChildBand)e.Row);

                    bAutoExpandingOrCollapsing = false;
                }
                else
                {
                    bAutoExpandingOrCollapsing = false;

                    if (e.Row.RowType == Infragistics.Controls.Grids.RowType.DataRow)
                        AutoCollapseAllChildRows((Row)e.Row);

                    if (e.Row.RowType == Infragistics.Controls.Grids.RowType.ColumnLayoutHeaderRow)
                        AutoCollapseAllChildBands((ChildBand)e.Row);

                    bAutoExpandingOrCollapsing = false;
                }
            }
        }

        private void xmiSelectAll_Click(object _sender, EventArgs _evArgs)
        {
            Infragistics.Controls.Grids.XamGrid xg = dmPDB.FindName(sContextMenuXamGrid) as XamGrid;
            if (xg != null)
            {               
                m_vm.sBusyText = ViewStrings.Processing;
                m_vm.bIsBusy = true;

                vSelectAllRowsInGrid(xg);
            }
            m_vm.bIsBusy = false;
        }
                         
        private void vSelectAllRowsInGrid(XamGrid _xg) {
            //_xg.Rows.SelectAll(); //todo: not working but should work according to docs.
            foreach (Row r in _xg.Rows)
            {
                _xg.SelectionSettings.SelectedRows.Add(r);
            }   
            //dataGrid.Rows[0].ChildBands[0].Rows.SelectAll();
            //((GroupByRow)dataGrid.Rows[0]).Rows.SelectAll();

            int i = _xg.SelectionSettings.SelectedRows.Count;
           


        }

        #region ContextMenu
        private void evhContextMenu_Opening(object _sender, OpeningEventArgs _evArgs)
        {
            // This method handles all of the context menus in the PDB.
            // NOTE: I also determine and remember the grid to which this context menu belongs for use in processing the clicked item.

            XamContextMenu xcm = (XamContextMenu)_sender;

            // Determine the type of control on which the context menu has been activated
            var oControlType = xcm.PlacementTargetResolved;

            switch (oControlType.GetType().ToString())
            {
                #region XamPivotGrid
                case "Infragistics.Controls.Grids.XamPivotGrid":
                    XamPivotGrid xpg = xcm.PlacementTargetResolved as XamPivotGrid;
                    // Process the appropriate menu
                    sContextMenuXamGrid = xpg.Name;

                    // if there are no data, disable menu items
                    if (xpg.DataRows.Count < 1)
                    {
                        foreach (XamMenuItem xmi in xcm.Items)
                        {
                            xmi.IsEnabled = false;
                            break;                                

                        }
                    }
                    else
                    {
                        foreach (XamMenuItem xmi in xcm.Items)
                        {
                            xmi.IsEnabled = true;
                            break;
                        }
                    }
                    break;
                #endregion XamPivotGrid

                #region XamGrid
                case "Infragistics.Controls.Grids.XamGrid":
                     XamGrid xg = xcm.PlacementTargetResolved as XamGrid;
                    // Process the appropriate menu
                    sContextMenuXamGrid = xg.Name;
                    switch (xg.Name)
                    {                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                
                        #region xgTestList
                        case "xgTestList":
                            // if there are no tests, cant' retreive test points or part tests
                            if (xg.Rows.Count < 1)
                            {
                                foreach (XamMenuItem xmi in xcm.Items)
                                {
                                    switch (xmi.Name)
                                    {
                                        case "xmiRetrievePartTests":
                                        case "xmiRetrieveTPDefns":
                                        case "xmiExportTheTests":
                                        case "xmiBatchFullRep":
                                        case "xmiBatchTPData":
                                            xmi.IsEnabled = false;
                                            break;
                                        case "xmiPersonalDatabase":
                                            foreach (XamMenuItem xmi2 in xmi.Items)
                                            {
                                                switch (xmi2.Name)
                                                {
                                                    case "xmiSaveAll":
                                                    case "xmiSaveSelected":
                                                    case "xmiDelete":
                                                        xmi2.IsEnabled = false;
                                                        break;
                                                    case "xmiOpenExisting":
                                                        xmi2.IsEnabled = true;
                                                        break;
                                                    default:
                                                        break;
                                                }
                                            }
                                            break;
                                        default:
                                            break;
                                    }
                                }
                                //_evArgs.Cancel = true;
                            }
                            else
                            {
                                foreach (XamMenuItem xmi in xcm.Items)
                                {
                                    switch (xmi.Name)
                                    {
                                        case "xmiRetrievePartTests":
                                        case "xmiRetrieveTPDefns":
                                        case "xmiExportTheTests":
                                        case "xmiBatchFullRep":
                                        case "xmiBatchTPData":
                                            xmi.IsEnabled = true;
                                            break;
                                        case "xmiPersonalDatabase":
                                            foreach (XamMenuItem xmi2 in xmi.Items)
                                            {
                                                xmi2.IsEnabled = true;
                                            }
                                            break;
                                        default:
                                            break;
                                    }
                                }
                            }
                            break;
                        #endregion xgTestList
                        #region xgTPList
                        case "xgTPList":                  
                            break;
                        #endregion xgTPList                       
                        default:
                            #region xgFullRep
                                                                                                                                                                                                                                                                                                    // is this a full report context menu
                    //if (xg.Name.StartsWith("FR_"))
                    //{
                    //    foreach (XamMenuItem xmi in xcm.Items)
                    //    {
                    //        switch (xmi.Name)
                    //        {
                    //            case "xmiExportTestDetailInfo":
                    //            case "xmiBatchDetailFullRep":
                    //                xmi.IsEnabled = false;
                    //                break;
                    //            case "xmiPersonalDatabase":
                    //                foreach (XamMenuItem xmi2 in xmi.Items)
                    //                {
                    //                    switch (xmi2.Name)
                    //                    {
                    //                        case "xmiSaveAll":
                    //                        case "xmiSaveSelected":
                    //                        case "xmiDelete":
                    //                            xmi2.IsEnabled = false;
                    //                            break;
                    //                        case "xmiOpenExisting":
                    //                            xmi2.IsEnabled = true;
                    //                            break;
                    //                        default:
                    //                            break;
                    //                    }
                    //                }
                    //                break;
                    //            default:
                    //                break;
                    //        }
                    //    }
                    //}
                            #endregion xgFullRep
                            break;
                    }
                    break;
                #endregion XamGrid
                ////////#region XamDataChart
                ////////case "WMAApp.Controls.XamDataChartEx":
                ////////    #region xdcTrendChart
                ////////    case "xdcTrendChart": 
                ////////        break;
                ////////    #endregion xdcTrendChart
                ////////#endregion XamDataChart
            }           
        }
        #endregion ContextMenu
        #endregion XamGrid General

        #region Utility
        private childItem FindVisualChild<childItem>(DependencyObject obj)
            where childItem : DependencyObject
        {

            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(obj); i++)
            {

                DependencyObject child = VisualTreeHelper.GetChild(obj, i);

                if (child != null && child is childItem)
                    return (childItem)child;
                else
                {
                    childItem childOfChild = FindVisualChild<childItem>(child);
                    if (childOfChild != null)
                        return childOfChild;
                }
            }

            return null;
        }
        #endregion Utility

        #endregion PDB

        private void radCurrent_Click(object sender, RoutedEventArgs e)
        {
            vShowDateControls(false);
        }

        private void radCustom_Click(object sender, RoutedEventArgs e)
        {
            vShowDateControls(true);
        }

        private void vShowDateControls(bool boShowCustom)
        {
            if (boShowCustom)
            {
                spResolution.Visibility = System.Windows.Visibility.Collapsed;
                spIntervals.Visibility = System.Windows.Visibility.Collapsed;

                spStartDate.Visibility = System.Windows.Visibility.Visible;
                spEndDate.Visibility = System.Windows.Visibility.Visible;
            }
            else
            {
                spResolution.Visibility = System.Windows.Visibility.Visible;
                spIntervals.Visibility = System.Windows.Visibility.Visible;

                spStartDate.Visibility = System.Windows.Visibility.Collapsed;
                spEndDate.Visibility = System.Windows.Visibility.Collapsed;
            }
        }

        private void radPrevious_Click(object sender, RoutedEventArgs e)
        {
            vShowDateControls(false);
        }

        private void radHour_Click(object sender, RoutedEventArgs e)
        {
            m_vm.sResolution = "Hour(s)";
        }

        private void radShift_Click(object sender, RoutedEventArgs e)
        {
            m_vm.sResolution = "Shift(s)";
        }

        private void radDay_Click(object sender, RoutedEventArgs e)
        {
            m_vm.sResolution = "Day(s)";
        }

        private void radWeek_Click(object sender, RoutedEventArgs e)
        {
            m_vm.sResolution = "Week(s)";
        }

        private void radMonth_Click(object sender, RoutedEventArgs e)
        {
            m_vm.sResolution = "Month(s)";
        }

        // when the timer elapses go get any new data
        void tmrDataRefresh_Tick(object sender, EventArgs e)
        {
            btnGetData_Click(null, null);
        }

        #endregion Methods

        private void chkRefresh_Click(object sender, RoutedEventArgs e)
        {
            if ((bool)chkRefresh.IsChecked)
            {
                tmrDataRefresh.Start();
            }
            else
            {
                tmrDataRefresh.Stop();
            }
        }

        private void NumericUpDown_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if ( m_vm != null )
                tmrDataRefresh.Interval = new TimeSpan( 0, 0, m_vm.iRefreshRate );
        }

    }

    #region Helper classes


    public class compTestStatusComparer : IComparer<object>
    {
        public int Compare(object _value1, object _value2)
        {
#if SUPPORT_UNDO_REDO
            ProductDataUR pd1 = _value1 as ProductDataUR;
            ProductDataUR pd2 = _value2 as ProductDataUR;
#else
            ProductData pd1 = _value1 as ProductData;
            ProductData pd2 = _value2 as ProductData;
#endif
            return pd1.sStatus.CompareTo(pd2.sStatus);
        }
    }

    public class convTestStatusToIcon : IValueConverter
    {   // Notes on unboundColumn binding:
        // "{Binding Value}" – this is the value returned from the ValueConverter
        // "{Binding ColumnKey}" – this is the column key of the unbound column
        // "{Binding RowData}" – the data object that the current row represents would represent


        const string sACCEPT_STATUS_CODE = "4"; // todo: store in global location later...
        const string sREJECT_STATUS_CODE = "5";

        public object Convert(object _value, Type _targetType, object _parameter, CultureInfo culture)
        {

            if (_value == null)
                return null;
#if SUPPORT_UNDO_REDO
            ProductDataUR pd = _value as ProductDataUR;
#else
            ProductData pd = _value as ProductData;
#endif

            //Get the root path for the XAP 
            //string src = Application.Current.Host.Source.ToString();
            //Get the application root, where 'ClientBin' is the known dir where the XAP is 
            //string appRoot = src.Substring(0, src.IndexOf("ClientBin"));
            //Create the image / uri 

            if (pd.sStatus == sACCEPT_STATUS_CODE)
            {
                return new Uri("/WMAApp;component/Assets/Images/Home/thumbs_up.png", UriKind.Relative);
                //return new Uri(appRoot + "/Assets/Images/Home/thumbs_up.png", UriKind.Absolute);
            }
            else if (pd.sStatus == sREJECT_STATUS_CODE)
            {
                return new Uri("/WMAApp;component/Assets/Images/Home/thumbs_down.png", UriKind.Relative);
                //return new Uri(appRoot + "/Assets/Images/Home/thumbs_down.png", UriKind.Absolute);
            }
            else
            {
                return null;
            }

        }
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return new NotImplementedException();
        }
    }
    #endregion Helper classes

}